@@ -1,39 +1,23 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
 """Patching matplotlib's FigureManager"""
 
-import os.path as osp
 
-from PyQt4.QtGui import QIcon, QCursor, QInputDialog, QMainWindow
-from PyQt4.QtCore import Qt, PYQT_VERSION_STR, SIGNAL, QObject
-
-# Local imports
-from spyderlib.config import get_icon
-
-def configure(backend=None, font_size=None, facecolor=None):
-    """Customizing matplotlib's parameters"""
-    from matplotlib import rcParams
-    if font_size is not None:
-        rcParams['font.size'] = font_size
-    rcParams["interactive"] = True # interactive mode
-    if backend is not None:
-        rcParams["backend"] = backend
-    if facecolor is not None:
-        rcParams['figure.facecolor'] = facecolor
-
-def apply(main=None, is_dockable=None):
-    """Monkey patching matplotlib Qt4 backend figures"""
-    if is_dockable is None:
-        is_dockable = lambda: False
-    
-    # Monkey patching matplotlib's figure manager for better integration
-    from matplotlib.backends import backend_qt4
-    from spyderlib.plugins.figure import MatplotlibFigure
+def set_backend(backend):
+    """Set matplotlib's backend: Qt4Agg, WXAgg, ..."""
     import matplotlib
+    matplotlib.use(backend)
+
+
+def apply():
+    """Monkey patching matplotlib Qt4 backend figures"""
+    from PyQt4.QtGui import QIcon, QCursor, QInputDialog, QMainWindow
+    from PyQt4.QtCore import Qt, PYQT_VERSION_STR, SIGNAL, QObject
+    from matplotlib.backends import backend_qt4
     
     # Class added to matplotlib to fix a bug with PyQt4 v4.6+
     class FigureWindow(QMainWindow):
@@ -52,18 +36,18 @@ def apply(main=None, is_dockable=None):
         Patching matplotlib...
         """
         def __init__(self, canvas, num):
+            import matplotlib
+            
             if backend_qt4.DEBUG:
                 print 'FigureManagerQT.%s' % backend_qt4.fn_name()
             backend_qt4.FigureManagerBase.__init__(self, canvas, num)
             self.canvas = canvas
             
-            if is_dockable():
-                self.window = MatplotlibFigure(main, canvas, num)
-            else:
-                self.window = FigureWindow()
-                self.window.setWindowTitle("Figure %d" % num)
+            self.window = FigureWindow()
+            self.window.setWindowTitle("Figure %d" % num)
             self.window.setAttribute(Qt.WA_DeleteOnClose)
-    
+
+            import os.path as osp
             image = osp.join(matplotlib.rcParams['datapath'],
                              'images', 'matplotlib.png' )
             self.window.setWindowIcon(QIcon(image))
@@ -81,15 +65,10 @@ def apply(main=None, is_dockable=None):
             QObject.connect(self.toolbar, SIGNAL("message"),
                     self.window.statusBar().showMessage)
     
-            if not is_dockable():
-                self.window.setCentralWidget(self.canvas)
+            self.window.setCentralWidget(self.canvas)
     
             if matplotlib.is_interactive():
-                if is_dockable():
-                    main.add_dockwidget(self.window)
-                    main.console.shell.setFocus()
-                else:
-                    self.window.show()
+                self.window.show()
     
             # attach a show method to the figure for pylab ease of use
             self.canvas.figure.show = lambda *args: self.window.show()
@@ -117,6 +96,7 @@ def apply(main=None, is_dockable=None):
         def _init_toolbar(self):
             super(NavigationToolbar2QT, self)._init_toolbar()
             if edit_parameters is None:
+                from spyderlib.config import get_icon
                 a = self.addAction(get_icon("options.svg"),
                                    'Customize', self.edit_parameters)
                 a.setToolTip('Edit curves line and axes parameters')
@@ -153,11 +133,7 @@ def apply(main=None, is_dockable=None):
             else:
                 super(NavigationToolbar2QT, self).edit_parameters()
         def save_figure(self):
-            if main is not None:
-                main.console.shell.restore_stds()
             super(NavigationToolbar2QT, self).save_figure()
-            if main is not None:
-                main.console.shell.redirect_stds()
         def set_cursor(self, cursor):
             if backend_qt4.DEBUG: print 'Set cursor' , cursor
             self.parent().setCursor(QCursor(backend_qt4.cursord[cursor]))
