@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -19,7 +19,7 @@ These plugins inherit the following classes
 # pylint: disable-msg=R0201
 
 from PyQt4.QtGui import (QDockWidget, QWidget, QFontDialog, QShortcut, QCursor,
-                         QKeySequence, QMainWindow, QApplication)
+                         QKeySequence, QMainWindow, QApplication, QAction, QVBoxLayout)
 from PyQt4.QtCore import SIGNAL, Qt, QObject
 
 import sys
@@ -30,16 +30,42 @@ STDOUT = sys.stdout
 # Local imports
 from spyderlib.utils.qthelpers import (toggle_actions, create_action,
                                        add_actions, translate)
-from spyderlib.config import CONF, get_font, set_font, get_icon
+from spyderlib.config import (CONF, get_font, set_font, get_icon,
+                              get_color_scheme, get_shortcut)
+from spyderlib.userconfig import NoDefault
+from spyderlib.plugins.configdialog import SpyderConfigPage
 from spyderlib.widgets.editor import CodeEditor
 from spyderlib.widgets.findreplace import FindReplace
+from spyderlib.widgets.browser import WebView
+    
+
+class PluginConfigPage(SpyderConfigPage):
+    """Plugin configuration dialog box page widget"""
+    def __init__(self, plugin, parent):
+        self.plugin = plugin
+        self.get_option = plugin.get_option
+        self.set_option = plugin.set_option
+        self.get_name = plugin.get_plugin_title
+        self.get_icon = plugin.get_plugin_icon
+        self.get_font = plugin.get_plugin_font
+        self.set_font = plugin.set_plugin_font
+        self.apply_settings = plugin.apply_plugin_settings
+        SpyderConfigPage.__init__(self, parent)
 
 
 class SpyderPluginMixin(object):
     """
     Useful methods to bind widgets to the main window
     See SpyderPluginWidget class for required widget interface
+    
+    Signals:
+        'option_changed'
+            Example:
+            self.emit(SIGNAL('option_changed'), 'show_all', checked)
+        'show_message(QString,int)'
     """
+    CONF_SECTION = None
+    CONFIGWIDGET_CLASS = None
     FLAGS = Qt.Window
     ALLOWED_AREAS = Qt.AllDockWidgetAreas
     LOCATION = Qt.LeftDockWidgetArea
@@ -50,14 +76,27 @@ class SpyderPluginMixin(object):
     def __init__(self, main):
         """Bind widget to a QMainWindow instance"""
         super(SpyderPluginMixin, self).__init__()
+        assert self.CONF_SECTION is not None
         self.main = main
-        self.menu_actions, self.toolbar_actions = self.get_plugin_actions()
+        self.default_margins = None
+        self.plugin_actions = self.get_plugin_actions()
         self.dockwidget = None
         self.ismaximized = False
-        QObject.connect(self, SIGNAL('option_changed'), self.option_changed)
+        self.isvisible = False
+        QObject.connect(self, SIGNAL('option_changed'), self.set_option)
         QObject.connect(self, SIGNAL('show_message(QString,int)'),
                         self.show_message)
         
+    def update_margins(self):
+        layout = self.layout()
+        if self.default_margins is None:
+            self.default_margins = layout.getContentsMargins()
+        if CONF.get('main', 'use_custom_margin', True):
+            margin = CONF.get('main', 'custom_margin', 0)
+            layout.setContentsMargins(*[margin]*4)
+        else:
+            layout.setContentsMargins(*self.default_margins)
+        
     def create_dockwidget(self):
         """Add to parent QMainWindow as a dock widget"""
         dock = QDockWidget(self.get_plugin_title(), self.main)#, self.FLAGS) -> bug in Qt 4.4
@@ -65,14 +104,18 @@ class SpyderPluginMixin(object):
         dock.setAllowedAreas(self.ALLOWED_AREAS)
         dock.setFeatures(self.FEATURES)
         dock.setWidget(self)
+        self.update_margins()
         self.connect(dock, SIGNAL('visibilityChanged(bool)'),
                      self.visibility_changed)
         self.dockwidget = dock
         self.refresh_plugin()
-        short = CONF.get(self.ID, "shortcut", None)
+        short = self.get_option("shortcut", None)
         if short is not None:
-            QShortcut(QKeySequence(short), self.main,
-                      lambda: self.visibility_changed(True))
+            shortcut = QShortcut(QKeySequence(short),
+                                 self.main, self.switch_to_plugin)
+            self.register_shortcut(shortcut, "_",
+                                   "Switch to %s" % self.CONF_SECTION,
+                                   default=short)
         return (dock, self.LOCATION)
     
     def create_mainwindow(self):
@@ -90,6 +133,41 @@ class SpyderPluginMixin(object):
         mainwindow.setCentralWidget(self)
         self.refresh_plugin()
         return mainwindow
+    
+    def create_configwidget(self, parent):
+        """Create configuration dialog box page widget"""
+        if self.CONFIGWIDGET_CLASS is not None:
+            configwidget = self.CONFIGWIDGET_CLASS(self, parent)
+            configwidget.initialize()
+            return configwidget
+
+    def apply_plugin_settings(self, options):
+        """Apply configuration file's plugin settings"""
+        raise NotImplementedError
+    
+    def register_shortcut(self, qaction_or_qshortcut, context, name,
+                          default=NoDefault):
+        """
+        Register QAction or QShortcut to Spyder main application,
+        with shortcut (context, name, default)
+        """
+        self.main.register_shortcut(qaction_or_qshortcut,
+                                    context, name, default)
+        
+    def register_widget_shortcuts(self, context, widget):
+        """
+        Register widget shortcuts
+        widget interface must have a method called 'get_shortcut_data'
+        """
+        for qshortcut, name, default in widget.get_shortcut_data():
+            self.register_shortcut(qshortcut, context, name, default)
+    
+    def switch_to_plugin(self):
+        """Switch to plugin
+        This method is called when pressing plugin's shortcut key"""
+        if not self.ismaximized:
+            self.dockwidget.show()
+        self.visibility_changed(True)
 
     def visibility_changed(self, enable):
         """DockWidget visibility has changed"""
@@ -100,18 +178,30 @@ class SpyderPluginMixin(object):
                 widget.setFocus()
         visible = self.dockwidget.isVisible() or self.ismaximized
         if self.DISABLE_ACTIONS_WHEN_HIDDEN:
-            toggle_actions(self.menu_actions, visible)
-            toggle_actions(self.toolbar_actions, visible)
+            toggle_actions(self.plugin_actions, visible)
         if visible:
             self.refresh_plugin() #XXX Is it a good idea?
+        self.isvisible = enable and visible
 
-    def option_changed(self, option, value):
+    def set_option(self, option, value):
         """
-        Change a plugin option in configuration file
+        Set a plugin option in configuration file
         Use a SIGNAL to call it, e.g.:
         self.emit(SIGNAL('option_changed'), 'show_all', checked)
         """
-        CONF.set(self.ID, option, value)
+        CONF.set(self.CONF_SECTION, option, value)
+
+    def get_option(self, option, default=NoDefault):
+        """Get a plugin option from configuration file"""
+        return CONF.get(self.CONF_SECTION, option, default)
+    
+    def get_plugin_font(self, option=None):
+        """Return plugin font option"""
+        return get_font(self.CONF_SECTION, option)
+    
+    def set_plugin_font(self, font, option=None):
+        """Set plugin font option"""
+        set_font(font, self.CONF_SECTION, option)
         
     def show_message(self, message, timeout=0):
         """Show message in main window's status bar"""
@@ -134,6 +224,15 @@ class SpyderPluginMixin(object):
         QApplication.restoreOverrideCursor()
         self.show_message(message, timeout=2000)
         QApplication.processEvents()
+        
+    def set_default_color_scheme(self, name='Spyder'):
+        """Set default color scheme (only once)"""
+        color_scheme_name = self.get_option('color_scheme_name', None)
+        if color_scheme_name is None:
+            names = CONF.get("color_schemes", "names")
+            if name not in names:
+                name = names[0]
+            self.set_option('color_scheme_name', name)
 
 
 class SpyderPluginWidget(QWidget, SpyderPluginMixin):
@@ -141,11 +240,9 @@ class SpyderPluginWidget(QWidget, SpyderPluginMixin):
     Spyder base widget class
     Spyder's widgets either inherit this class or reimplement its interface
     """
-    ID = None
     def __init__(self, parent):
         QWidget.__init__(self, parent)
         SpyderPluginMixin.__init__(self, parent)
-        assert self.ID is not None
         self.setWindowTitle(self.get_plugin_title())
         
     def get_plugin_title(self):
@@ -158,11 +255,12 @@ class SpyderPluginWidget(QWidget, SpyderPluginMixin):
     
     def get_plugin_icon(self):
         """
-        Return plugin icon name (e.g.: qt.png) or QIcon instance
-        Note: this is required only for plugins creating a main window
+        Return plugin icon (QIcon instance)
+        Note: this is required for plugins creating a main window
               (see SpyderPluginMixin.create_mainwindow)
+              and for configuration dialog widgets creation
         """
-        return 'qt.png'
+        return get_icon('qt.png')
     
     def get_focus_widget(self):
         """
@@ -185,72 +283,116 @@ class SpyderPluginWidget(QWidget, SpyderPluginMixin):
     
     def get_plugin_actions(self):
         """
-        Setup actions and return a tuple (menu_actions, toolbar_actions)
-        (each tuple element contains a list of QAction objects or None)
+        Return a list of actions related to plugin
+        Note: these actions will be enabled when plugin's dockwidget is visible
+              and they will be disabled when it's hidden
         """
         raise NotImplementedError
+    
+    def register_plugin(self):
+        """Register plugin in Spyder's main window"""
+        raise NotImplementedError
 
 
-class ReadOnlyEditor(SpyderPluginWidget):
+class RichText(QWidget):
     """
-    Read-only editor plugin widget
-    (see example of child class in inspector.py)
+    WebView widget with find dialog
     """
     def __init__(self, parent):
-        SpyderPluginWidget.__init__(self, parent)
+        QWidget.__init__(self, parent)
+        
+        self.webview = WebView(self)
+        self.find_widget = FindReplace(self)
+        self.find_widget.set_editor(self.webview)
+        self.find_widget.hide()
+        
+        layout = QVBoxLayout()
+        layout.setContentsMargins(0, 0, 0, 0)
+        layout.addWidget(self.webview)
+        layout.addWidget(self.find_widget)
+        self.setLayout(layout)
+        
+class PlainText(QWidget):
+    """
+    Read-only editor widget with find dialog
+    """
+    def __init__(self, parent):
+        QWidget.__init__(self, parent)
+        self.editor = None
 
         # Read-only editor
         self.editor = CodeEditor(self)
         self.editor.setup_editor(linenumbers=False, language='py',
-                                 code_folding=True, scrollflagarea=False)
+                                 scrollflagarea=False)
         self.connect(self.editor, SIGNAL("focus_changed()"),
                      lambda: self.emit(SIGNAL("focus_changed()")))
         self.editor.setReadOnly(True)
-        self.editor.set_font( get_font(self.ID) )
-        self.editor.toggle_wrap_mode( CONF.get(self.ID, 'wrap') )
-        
-        # Add entries to read-only editor context-menu
-        font_action = create_action(self, translate("Editor", "&Font..."), None,
-                                    'font.png',
-                                    translate("Editor", "Set font style"),
-                                    triggered=self.change_font)
-        wrap_action = create_action(self, translate("Editor", "Wrap lines"),
-                                    toggled=self.toggle_wrap_mode)
-        wrap_action.setChecked( CONF.get(self.ID, 'wrap') )
-        self.editor.readonly_menu.addSeparator()
-        add_actions(self.editor.readonly_menu, (font_action, wrap_action))
         
         # Find/replace widget
         self.find_widget = FindReplace(self)
         self.find_widget.set_editor(self.editor)
         self.find_widget.hide()
         
+        layout = QVBoxLayout()
+        layout.setContentsMargins(0, 0, 0, 0)
+        layout.addWidget(self.editor)
+        layout.addWidget(self.find_widget)
+        self.setLayout(layout)
+        
+class RichAndPlainText(SpyderPluginWidget):
+    """
+    Plugin widget to view docstrings as html or
+    plain text
+    (see example of child class in inspector.py)
+    """
+    def __init__(self, parent):
+        SpyderPluginWidget.__init__(self, parent)
+        
+        self.plain_text = PlainText(self)
+        self.rich_text = RichText(self)
+        
+        color_scheme = get_color_scheme(self.get_option('color_scheme_name'))
+        self.plain_text.editor.set_font(self.get_plugin_font(), color_scheme)
+        self.plain_text.editor.toggle_wrap_mode(self.get_option('wrap'))
+        
+        # Add entries to read-only editor context-menu
+        font_action = create_action(self, translate("Editor", "&Font..."), None,
+                                    'font.png',
+                                    translate("Editor", "Set font style"),
+                                    triggered=self.change_font)
+        self.wrap_action = create_action(self,
+                                         translate("Editor", "Wrap lines"),
+                                         toggled=self.toggle_wrap_mode)
+        self.wrap_action.setChecked(self.get_option('wrap'))
+        self.plain_text.editor.readonly_menu.addSeparator()
+        add_actions(self.plain_text.editor.readonly_menu,
+                    (font_action, self.wrap_action))
+        
         # <!> Layout will have to be implemented in child class!
+        
+    @property
+    def find_widget(self):
+        if self.plain_text.isVisible():
+            return self.plain_text.find_widget
+        else:
+            return self.rich_text.find_widget
     
     def get_focus_widget(self):
         """
         Return the widget to give focus to when
         this plugin's dockwidget is raised on top-level
         """
-        return self.editor
-            
-    def get_plugin_actions(self):
-        """Setup and return actions"""
-        return (None, None)
-        
-    def closing_plugin(self, cancelable=False):
-        """Perform actions before parent main window is closed"""
-        return True
+        return self.rich_text
         
     def change_font(self):
         """Change console font"""
-        font, valid = QFontDialog.getFont(get_font(self.ID), self,
+        font, valid = QFontDialog.getFont(get_font(self.CONF_SECTION), self,
                                       translate("Editor", "Select a new font"))
         if valid:
-            self.editor.set_font(font)
-            set_font(font, self.ID)
+            self.plain_text.editor.set_font(font)
+            set_font(font, self.CONF_SECTION)
             
     def toggle_wrap_mode(self, checked):
         """Toggle wrap mode"""
-        self.editor.toggle_wrap_mode(checked)
-        CONF.set(self.ID, 'wrap', checked)
+        self.plain_text.editor.toggle_wrap_mode(checked)
+        self.set_option('wrap', checked)
