@@ -1,13 +1,13 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
 """Console History Plugin"""
 
 from PyQt4.QtGui import (QVBoxLayout, QFontDialog, QInputDialog, QToolButton,
-                         QMenu)
+                         QMenu, QFontComboBox, QGroupBox)
 from PyQt4.QtCore import SIGNAL
 
 import os.path as osp
@@ -18,30 +18,69 @@ STDOUT = sys.stdout
 
 # Local imports
 from spyderlib.utils import encoding
-from spyderlib.config import CONF, get_font, get_icon, set_font
+from spyderlib.config import get_icon, CONF, get_color_scheme
 from spyderlib.utils.qthelpers import (create_action, create_toolbutton,
                                        add_actions)
 from spyderlib.widgets.tabs import Tabs
 from spyderlib.widgets.editor import CodeEditor
 from spyderlib.widgets.findreplace import FindReplace
-from spyderlib.plugins import SpyderPluginWidget
+from spyderlib.plugins import SpyderPluginWidget, PluginConfigPage
+
+
+class HistoryConfigPage(PluginConfigPage):
+    def setup_page(self):
+        settings_group = QGroupBox(self.tr("Settings"))
+        hist_spin = self.create_spinbox(
+                            self.tr("History depth: "), self.tr(" entries"),
+                            'max_entries', min_=10, max_=10000, step=10,
+                            tip=self.tr("Set maximum line count"))
+
+        sourcecode_group = QGroupBox(self.tr("Source code"))
+        wrap_mode_box = self.create_checkbox(self.tr("Wrap lines"), 'wrap')
+        font_group = self.create_fontgroup(option=None,
+                                    text=self.tr("Font style"),
+                                    fontfilters=QFontComboBox.MonospacedFonts)
+        names = CONF.get('color_schemes', 'names')
+        choices = zip(names, names)
+        cs_combo = self.create_combobox(self.tr("Syntax color scheme: "),
+                                        choices, 'color_scheme_name')
+
+        settings_layout = QVBoxLayout()
+        settings_layout.addWidget(hist_spin)
+        settings_group.setLayout(settings_layout)
+
+        sourcecode_layout = QVBoxLayout()
+        sourcecode_layout.addWidget(wrap_mode_box)
+        sourcecode_layout.addWidget(cs_combo)
+        sourcecode_group.setLayout(sourcecode_layout)
+        
+        vlayout = QVBoxLayout()
+        vlayout.addWidget(settings_group)
+        vlayout.addWidget(font_group)
+        vlayout.addWidget(sourcecode_group)
+        vlayout.addStretch(1)
+        self.setLayout(vlayout)
 
 
 class HistoryLog(SpyderPluginWidget):
     """
     History log widget
     """
-    ID = 'historylog'
+    CONF_SECTION = 'historylog'
+    CONFIGWIDGET_CLASS = HistoryConfigPage
     def __init__(self, parent):
         self.tabwidget = None
         self.menu_actions = None
         self.dockviewer = None
+        self.wrap_action = None
         
         self.editors = []
         self.filenames = []
         self.icons = []
         
         SpyderPluginWidget.__init__(self, parent)
+
+        self.set_default_color_scheme()
         
         layout = QVBoxLayout()
         self.tabwidget = Tabs(self, self.menu_actions)
@@ -53,7 +92,8 @@ class HistoryLog(SpyderPluginWidget):
 
         # Menu as corner widget
         options_button = create_toolbutton(self, text=self.tr("Options"),
-                                           icon=get_icon('tooloptions.png'))
+                                           icon=get_icon('tooloptions.png'),
+                                           text_beside_icon=True)
         options_button.setPopupMode(QToolButton.InstantPopup)
         menu = QMenu(self)
         add_actions(menu, self.menu_actions)
@@ -63,6 +103,8 @@ class HistoryLog(SpyderPluginWidget):
         # Find/replace widget
         self.find_widget = FindReplace(self)
         self.find_widget.hide()
+        self.register_widget_shortcuts("Editor", self.find_widget)
+        
         layout.addWidget(self.find_widget)
         
         self.setLayout(layout)
@@ -72,6 +114,10 @@ class HistoryLog(SpyderPluginWidget):
         """Return widget title"""
         return self.tr('History log')
     
+    def get_plugin_icon(self):
+        """Return widget icon"""
+        return get_icon('history.png')
+    
     def get_focus_widget(self):
         """
         Return the widget to give focus to when
@@ -92,7 +138,7 @@ class HistoryLog(SpyderPluginWidget):
         self.find_widget.set_editor(editor)
         
     def get_plugin_actions(self):
-        """Setup actions"""
+        """Return a list of actions related to plugin"""
         history_action = create_action(self, self.tr("History..."),
                                        None, 'history.png',
                                        self.tr("Set history maximum entries"),
@@ -100,11 +146,38 @@ class HistoryLog(SpyderPluginWidget):
         font_action = create_action(self, self.tr("&Font..."), None,
                                     'font.png', self.tr("Set shell font style"),
                                     triggered=self.change_font)
-        wrap_action = create_action(self, self.tr("Wrap lines"),
+        self.wrap_action = create_action(self, self.tr("Wrap lines"),
                                     toggled=self.toggle_wrap_mode)
-        wrap_action.setChecked( CONF.get(self.ID, 'wrap') )
-        self.menu_actions = [history_action, font_action, wrap_action]
-        return (self.menu_actions, None)
+        self.wrap_action.setChecked( self.get_option('wrap') )
+        self.menu_actions = [history_action, font_action, self.wrap_action]
+        return self.menu_actions
+    
+    def register_plugin(self):
+        """Register plugin in Spyder's main window"""
+        self.connect(self, SIGNAL('focus_changed()'),
+                     self.main.plugin_focus_changed)
+        self.main.add_dockwidget(self)
+#        self.main.console.set_historylog(self)
+        self.connect(self.main.console.shell, SIGNAL("refresh()"),
+                     self.refresh_plugin)
+
+    def apply_plugin_settings(self, options):
+        """Apply configuration file's plugin settings"""
+        color_scheme_n = 'color_scheme_name'
+        color_scheme_o = get_color_scheme(self.get_option(color_scheme_n))
+        font_n = 'plugin_font'
+        font_o = self.get_plugin_font()
+        wrap_n = 'wrap'
+        wrap_o = self.get_option(wrap_n)
+        self.wrap_action.setChecked(wrap_o)
+        for editor in self.editors:
+            if font_n in options:
+                scs = color_scheme_o if color_scheme_n in options else None
+                editor.set_font(font_o, scs)
+            elif color_scheme_n in options:
+                editor.set_color_scheme(color_scheme_o)
+            if wrap_n in options:
+                editor.toggle_wrap_mode(wrap_o)
         
     #------ Private API --------------------------------------------------------
     def move_tab(self, index_from, index_to):
@@ -136,12 +209,13 @@ class HistoryLog(SpyderPluginWidget):
             language = 'bat'
             icon = get_icon('cmdprompt.png')
         editor.setup_editor(linenumbers=False, language=language,
-                            code_folding=True, scrollflagarea=False)
+                            scrollflagarea=False)
         self.connect(editor, SIGNAL("focus_changed()"),
                      lambda: self.emit(SIGNAL("focus_changed()")))
         editor.setReadOnly(True)
-        editor.set_font( get_font(self.ID) )
-        editor.toggle_wrap_mode( CONF.get(self.ID, 'wrap') )
+        color_scheme = get_color_scheme(self.get_option('color_scheme_name'))
+        editor.set_font( self.get_plugin_font(), color_scheme )
+        editor.toggle_wrap_mode( self.get_option('wrap') )
 
         text, _ = encoding.read(filename)
         editor.set_text(text)
@@ -172,19 +246,19 @@ class HistoryLog(SpyderPluginWidget):
         "Change history max entries"""
         depth, valid = QInputDialog.getInteger(self, self.tr('History'),
                                        self.tr('Maximum entries'),
-                                       CONF.get(self.ID, 'max_entries'),
+                                       self.get_option('max_entries'),
                                        10, 10000)
         if valid:
-            CONF.set(self.ID, 'max_entries', depth)
+            self.set_option('max_entries', depth)
         
     def change_font(self):
         """Change console font"""
-        font, valid = QFontDialog.getFont(get_font(self.ID),
+        font, valid = QFontDialog.getFont(self.get_plugin_font(),
                        self, self.tr("Select a new font"))
         if valid:
             for editor in self.editors:
                 editor.set_font(font)
-            set_font(font, self.ID)
+            self.set_plugin_font(font)
             
     def toggle_wrap_mode(self, checked):
         """Toggle wrap mode"""
@@ -192,4 +266,4 @@ class HistoryLog(SpyderPluginWidget):
             return
         for editor in self.editors:
             editor.toggle_wrap_mode(checked)
-        CONF.set(self.ID, 'wrap', checked)
+        self.set_option('wrap', checked)
