@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -31,17 +31,26 @@ def listdict2envdict(listdict):
             listdict[key] = os.path.pathsep.join(listdict[key])
     return listdict
 
-class EnvDialog(DictEditor):
-    """Environment variables Dialog"""
-    def __init__(self):
-        super(EnvDialog, self).__init__(envdict2listdict( dict(os.environ) ),
-                                        title="os.environ", width=600,
-                                        icon='environ.png')
+class RemoteEnvDialog(DictEditor):
+    """Remote process environment variables Dialog"""
+    def __init__(self, get_environ_func, set_environ_func):
+        DictEditor.__init__(self, envdict2listdict(get_environ_func()),
+                            title="os.environ", width=600, icon='environ.png')
+        self.set_environ = set_environ_func
     def accept(self):
         """Reimplement Qt method"""
-        os.environ = listdict2envdict( self.get_copy() )
+        self.set_environ(listdict2envdict(self.get_value()))
         QDialog.accept(self)
 
+class EnvDialog(RemoteEnvDialog):
+    """Environment variables Dialog"""
+    def __init__(self):
+        def get_environ_func():
+            return dict(os.environ)
+        def set_environ_func(env):
+            os.environ = env
+        RemoteEnvDialog.__init__(self, get_environ_func, set_environ_func)
+
 
 try:
     #---- Windows platform
@@ -61,7 +70,7 @@ try:
                 break
         return envdict2listdict(reg)
     
-    def set_user_env(reg):
+    def set_user_env(reg, parent=None):
         """Set HKCU (current user) environment variables"""
         reg = listdict2envdict(reg)
         types = dict()
@@ -81,7 +90,7 @@ try:
             SendMessageTimeout(HWND_BROADCAST, WM_SETTINGCHANGE, 0,
                                "Environment", SMTO_ABORTIFHUNG, 5000)
         except ImportError:
-            QMessageBox.warning(self,
+            QMessageBox.warning(parent,
                 translate("WinUserEnvDialog", "Warning"),
                 translate("WinUserEnvDialog",
                           "Module <b>pywin32 was not found</b>.<br>"
@@ -91,8 +100,9 @@ try:
     class WinUserEnvDialog(DictEditor):
         """Windows User Environment Variables Editor"""
         def __init__(self, parent=None):
-            super(WinUserEnvDialog, self).__init__(get_user_env(),
-               title="HKEY_CURRENT_USER\Environment", width=600)
+            DictEditor.__init__(self, get_user_env(),
+                                title="HKEY_CURRENT_USER\Environment",
+                                width=600)
             if parent is None:
                 parent = self
             QMessageBox.warning(parent,
@@ -110,7 +120,7 @@ try:
             
         def accept(self):
             """Reimplement Qt method"""
-            set_user_env( listdict2envdict(self.get_copy()) )
+            set_user_env( listdict2envdict(self.get_value()), parent=self )
             QDialog.accept(self)
 
 except ImportError:
@@ -121,9 +131,10 @@ except ImportError:
 def main():
     """Run Windows environment variable editor"""
     from spyderlib.utils.qthelpers import qapplication
-    _app = qapplication()
+    app = qapplication()
     dialog = WinUserEnvDialog()
-    dialog.exec_()
+    dialog.show()
+    app.exec_()
 
 if __name__ == "__main__":
     main()
