@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -9,14 +9,14 @@
 import os.path as osp
 import os, webbrowser
 
-from PyQt4.QtGui import (QAction, QStyle, QWidget, QIcon, QApplication,
-                         QVBoxLayout, QHBoxLayout, QLineEdit, QLabel,
-                         QKeySequence, QToolButton, QKeyEvent, QMenu)
+from PyQt4.QtGui import (QAction, QStyle, QWidget, QIcon, QApplication, QLabel,
+                         QVBoxLayout, QHBoxLayout, QLineEdit, QKeyEvent, QMenu,
+                         QKeySequence, QToolButton)
 from PyQt4.QtCore import (SIGNAL, QVariant, QObject, Qt, QLocale, QTranslator,
                           QLibraryInfo)
 
 # Local import
-from spyderlib.config import get_icon, DATA_PATH
+from spyderlib.config import get_icon, DATA_PATH, get_spyderplugins, PLUGIN_PATH
 from spyderlib.utils import programs
 
 # Note: How to redirect a signal from widget *a* to widget *b* ?
@@ -38,26 +38,37 @@ def qapplication(translate=True):
     Return QApplication instance
     Creates it if it doesn't already exist
     """
-    global QAPPLICATION, TRANSLATORS
+    global QAPPLICATION
     if QApplication.startingUp():
         QAPPLICATION = QApplication([])
         if translate:
-            locale = QLocale.system().name()
-            # Qt-specific translator
-            qt_translator = QTranslator()
-            TRANSLATORS.append(qt_translator) # Keep reference alive
-            paths = QLibraryInfo.location(QLibraryInfo.TranslationsPath)
-            if qt_translator.load("qt_"+locale, paths):
-                QAPPLICATION.installTranslator(qt_translator)
-            # Spyder-specific translator
-            app_translator = QTranslator()
-            TRANSLATORS.append(app_translator) # Keep reference alive
-            if app_translator.load("spyder_"+locale, DATA_PATH):
-                QAPPLICATION.installTranslator(app_translator)
+            install_translators(QAPPLICATION)
     elif QAPPLICATION is None:
         QAPPLICATION = QApplication.instance()
     return QAPPLICATION
 
+def install_translators(qapp):
+    """Install translators to the QApplication instance"""
+    global TRANSLATORS
+    locale = QLocale.system().name()
+    # Qt-specific translator
+    qt_translator = QTranslator()
+    TRANSLATORS.append(qt_translator) # Keep reference alive
+    paths = QLibraryInfo.location(QLibraryInfo.TranslationsPath)
+    if qt_translator.load("qt_"+locale, paths):
+        qapp.installTranslator(qt_translator)
+    # Spyder-specific translator
+    app_translator = QTranslator()
+    TRANSLATORS.append(app_translator) # Keep reference alive
+    if app_translator.load("spyder_"+locale, DATA_PATH):
+        qapp.installTranslator(app_translator)
+    # Load 3rd-party plugin translators
+    for modname in get_spyderplugins(prefix='p_', extension='.py'):
+        plugin_translator = QTranslator()
+        TRANSLATORS.append(plugin_translator)
+        if plugin_translator.load(modname+"_"+locale, PLUGIN_PATH):
+            qapp.installTranslator(plugin_translator)
+
 def translate(context, string):
     """Translation"""
     return QApplication.translate(context, string)
@@ -112,9 +123,9 @@ def restore_keyevent(event):
     shift = modifiers & Qt.ShiftModifier
     return event, text, key, ctrl, shift
 
-def create_toolbutton(parent, icon=None, text=None,
-                      triggered=None, tip=None, toggled=None, shortcut=None,
-                      autoraise=True, text_beside_icon=True):
+def create_toolbutton(parent, text=None, shortcut=None, icon=None, tip=None,
+                      toggled=None, triggered=None,
+                      autoraise=True, text_beside_icon=False):
     """Create a QToolButton"""
     button = QToolButton(parent)
     if text is not None:
@@ -137,6 +148,17 @@ def create_toolbutton(parent, icon=None, text=None,
         button.setShortcut(shortcut)
     return button
 
+def action2button(action, autoraise=True, text_beside_icon=False, parent=None):
+    """Create a QToolButton directly from a QAction object"""
+    if parent is None:
+        parent = action.parent()
+    button = QToolButton(parent)
+    button.setDefaultAction(action)
+    button.setAutoRaise(autoraise)
+    if text_beside_icon:
+        button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+    return button
+
 def toggle_actions(actions, enable):
     """Enable/disable actions"""
     if actions is not None:
@@ -242,16 +264,50 @@ def create_program_action(parent, text, icon, name, nt_name=None):
         return create_action(parent, text, icon=icon,
                              triggered=lambda: programs.run_program(name))
         
-def create_python_gui_script_action(parent, text, icon,
-                                    package, module, args=''):
+def create_python_script_action(parent, text, icon, package, module, args=[]):
     """Create action to run a GUI based Python script"""
     if isinstance(icon, basestring):
         icon = get_icon(icon)
-    if programs.is_python_gui_script_installed(package, module):
+    if programs.python_script_exists(package, module):
         return create_action(parent, text, icon=icon,
                              triggered=lambda:
-                             programs.run_python_gui_script(package, module,
-                                                            args))
+                             programs.run_python_script(package, module, args))
+
+
+class DialogManager(QObject):
+    """
+    Object that keep references to non-modal dialog boxes for another QObject,
+    typically a QMainWindow or any kind of QWidget
+    """
+    def __init__(self):
+        QObject.__init__(self)
+        self.dialogs = {}
+        
+    def show(self, dialog):
+        """Generic method to show a non-modal dialog and keep reference
+        to the Qt C++ object"""
+        for dlg in self.dialogs.values():
+            if unicode(dlg.windowTitle()) == unicode(dialog.windowTitle()):
+                dlg.show()
+                dlg.raise_()
+                break
+        else:
+            dialog.show()
+            self.dialogs[id(dialog)] = dialog
+            self.connect(dialog, SIGNAL('accepted()'),
+                         lambda eid=id(dialog): self.dialog_finished(eid))
+            self.connect(dialog, SIGNAL('rejected()'),
+                         lambda eid=id(dialog): self.dialog_finished(eid))
+        
+    def dialog_finished(self, dialog_id):
+        """Manage non-modal dialog boxes"""
+        return self.dialogs.pop(dialog_id)
+    
+    def close_all(self):
+        """Close all opened dialog boxes"""
+        for dlg in self.dialogs.values():
+            dlg.reject()
+
         
 def get_std_icon(name, size=None):
     """Get standard platform icon
@@ -277,7 +333,7 @@ class ShowStdIcons(QWidget):
     Dialog showing standard icons
     """
     def __init__(self, parent):
-        super(ShowStdIcons, self).__init__(parent)
+        QWidget.__init__(self, parent)
         layout = QHBoxLayout()
         row_nb = 14
         cindex = 0
