@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -88,15 +88,22 @@ def get_idx_rect(index_list):
 
 class ArrayModel(QAbstractTableModel):
     """Array Editor Table Model"""
-    def __init__(self, data, changes,
-                 format="%.3f", xy_mode=False, readonly=False, parent=None):
-        super(ArrayModel, self).__init__()
+    def __init__(self, data, format="%.3f",
+                 xy_mode=False, readonly=False, parent=None):
+        QAbstractTableModel.__init__(self)
 
         self.dialog = parent
-        self.changes = changes
+        self.changes = {}
         self.readonly = readonly
         self.test_array = np.array([0], dtype=data.dtype)
 
+        # for complex numbers, shading will be based on absolute value
+        # but for all other types it will be the real part
+        if data.dtype in (np.complex64, np.complex128):
+            self.color_func = np.abs
+        else:
+            self.color_func = np.real
+        
         # Backgroundcolor settings
         huerange = [.66, .99] # Hue
         self.sat = .7 # Saturation
@@ -108,8 +115,8 @@ class ArrayModel(QAbstractTableModel):
         self._xy = xy_mode
         
         try:
-            self.vmin = data.min()
-            self.vmax = data.max()
+            self.vmin = self.color_func(data).min()
+            self.vmax = self.color_func(data).max()
             if self.vmax == self.vmin:
                 self.vmin -= 1
             self.hue0 = huerange[0]
@@ -165,7 +172,8 @@ class ArrayModel(QAbstractTableModel):
         elif role == Qt.TextAlignmentRole:
             return QVariant(int(Qt.AlignCenter|Qt.AlignVCenter))
         elif role == Qt.BackgroundColorRole and self.bgcolor_enabled:
-            hue = self.hue0+self.dhue*(self.vmax-value)/(self.vmax-self.vmin)
+            hue = self.hue0+self.dhue*(self.vmax-self.color_func(value))/(self.vmax-self.vmin)
+            hue = float(np.abs(hue))
             color = QColor.fromHsvF(hue, self.sat, self.val, self.alp)
             return QVariant(color)
         elif role == Qt.FontRole:
@@ -245,7 +253,7 @@ class ArrayModel(QAbstractTableModel):
 class ArrayDelegate(QItemDelegate):
     """Array Editor Item Delegate"""
     def __init__(self, dtype, parent=None):
-        super(ArrayDelegate, self).__init__(parent)
+        QItemDelegate.__init__(self, parent)
         self.dtype = dtype
 
     def createEditor(self, parent, option, index):
@@ -366,10 +374,8 @@ class ArrayEditorWidget(QWidget):
             self.old_data_shape = self.data.shape
             self.data.shape = (1, 1)
 
-        self.changes = {}
-       
         format = SUPPORTED_FORMATS.get(data.dtype.name, '%s')
-        self.model = ArrayModel(self.data, self.changes, format=format,
+        self.model = ArrayModel(self.data, format=format,
                                 xy_mode=xy, readonly=readonly, parent=self)
         self.view = ArrayView(self, self.model, data.dtype, data.shape)
         
@@ -396,7 +402,7 @@ class ArrayEditorWidget(QWidget):
         
     def accept_changes(self):
         """Accept changes"""
-        for (i, j), value in self.changes.iteritems():
+        for (i, j), value in self.model.changes.iteritems():
             self.data[i, j] = value
         if self.old_data_shape is not None:
             self.data.shape = self.old_data_shape
@@ -427,13 +433,15 @@ class ArrayEditorWidget(QWidget):
 class ArrayEditor(QDialog):
     """Array Editor Dialog"""    
     def __init__(self, parent=None):
-        super(ArrayEditor, self).__init__(parent)
+        QDialog.__init__(self, parent)
+        self.data = None
     
     def setup_and_check(self, data, title='', xy=False, readonly=False):
         """
         Setup ArrayEditor:
         return False if data is not supported, True otherwise
         """
+        self.data = data
         self.arraywidget = None
         self.is_record_array = data.dtype.names is not None
         if data.size == 0:
@@ -454,8 +462,8 @@ class ArrayEditor(QDialog):
         self.layout = QGridLayout()
         self.setLayout(self.layout)
         self.setWindowIcon(get_icon('arredit.png'))
-        title = self.tr("Array editor") + \
-                "%s" % (" - "+str(title) if str(title) else "")
+        if not title:
+            title = self.tr("Array editor")
         if readonly:
             title += ' (' + self.tr('read only') + ')'
         self.setWindowTitle(title)
@@ -515,6 +523,10 @@ class ArrayEditor(QDialog):
         for index in range(self.stack.count()):
             self.stack.widget(index).accept_changes()
         QDialog.accept(self)
+        
+    def get_value(self):
+        """Return modified array -- this is *not* a copy"""
+        return self.data
 
     def error(self, message):
         """An error occured, closing the dialog box"""
@@ -537,11 +549,13 @@ def aedit(data, title="", xy=False, readonly=False, parent=None):
     (instantiate a new QApplication if necessary,
     so it can be called directly from the interpreter)
     """
-    _app = qapplication()
+    app = qapplication()
     dialog = ArrayEditor(parent)
     if dialog.setup_and_check(data, title, xy=xy, readonly=readonly):
-        if dialog.exec_():
-            return data
+        dialog.show()
+        app.exec_()
+        if dialog.result():
+            return dialog.get_value()
 
 
 def test():
