@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -16,7 +16,7 @@ Dictionary Editor Widget and Dialog based on PyQt4
 # pylint: disable-msg=R0911
 # pylint: disable-msg=R0201
 
-import re, os
+import re, os, sys
 from PyQt4.QtGui import (QMessageBox, QTableView, QItemDelegate, QLineEdit,
                          QVBoxLayout, QWidget, QColor, QDialog, QDateEdit,
                          QDialogButtonBox, QMenu, QInputDialog, QDateTimeEdit,
@@ -38,6 +38,7 @@ class FakeObject(object):
     pass
 try:
     from numpy import ndarray
+    from numpy import array, matrix #@UnusedImport (object eval)
     from spyderlib.widgets.arrayeditor import ArrayEditor
 except ImportError:
     class ndarray(FakeObject):
@@ -47,6 +48,7 @@ except ImportError:
 #----PIL Images support
 try:
     from PIL.Image import Image
+    import PIL.Image
 except:
     class Image(FakeObject):
         """Fake PIL Image"""
@@ -86,6 +88,8 @@ COLORS = {
 
 def get_color_name(value):
     """Return color name depending on value type"""
+    if not is_known_type(value):
+        return "#7755aa"
     for typ, name in COLORS.iteritems():
         if isinstance(value, typ):
             return name
@@ -132,26 +136,32 @@ def try_to_eval(value):
     except (NameError, SyntaxError, ImportError):
         return value
     
-def display_to_value(value, default_value):
+def display_to_value(value, default_value, ignore_errors=True):
     """Convert back to value"""
     value = unicode(value.toString())
     try:
         if isinstance(default_value, str):
             value = str(value)
-        elif isinstance(default_value, (bool, list, dict, tuple)):
-            value = eval(value)
         elif isinstance(default_value, float):
             value = float(value)
         elif isinstance(default_value, int):
-            value = int(value)
+            try:
+                value = int(value)
+            except ValueError:
+                value = float(value)
         elif isinstance(default_value, datetime.datetime):
             value = datestr_to_datetime(value)
         elif isinstance(default_value, datetime.date):
             value = datestr_to_datetime(value).date()
-        else:
+        elif ignore_errors:
             value = try_to_eval(value)
+        else:
+            value = eval(value)
     except (ValueError, SyntaxError):
-        value = try_to_eval(value)
+        if ignore_errors:
+            value = try_to_eval(value)
+        else:
+            raise
     return value
 
 def get_size(item):
@@ -165,17 +175,28 @@ def get_size(item):
     else:
         return 1
 
+def get_type_string(item):
+    """Return type string of an object"""
+    if isinstance(item, ndarray):
+        return item.dtype.name
+    elif isinstance(item, Image):
+        return "Image"
+    else:
+        found = re.findall(r"<type '([\S]*)'>", str(type(item)))
+        if found:
+            return found[0]
+
 def get_type(item):
     """Return type of an item"""
-    found = re.findall(r"<type '([\S]*)'>", str(type(item)))
-    text = unicode(translate('DictEditor', 'unknown')) \
-           if not found else found[0]
-    if isinstance(item, ndarray):
-        text = item.dtype.name
-    if isinstance(item, Image):
-        text = "Image"
+    text = get_type_string(item)
+    if text is None:
+        text = unicode(translate('DictEditor', 'unknown'))
     return text[text.find('.')+1:]
 
+def is_known_type(item):
+    """Return True if object has a known type"""
+    return get_type_string(item) is not None
+
 
 class ReadOnlyDictModel(QAbstractTableModel):
     """DictEditor Read-Only Table Model"""
@@ -225,7 +246,7 @@ class ReadOnlyDictModel(QAbstractTableModel):
             if not self.names:
                 self.header0 = translate("DictEditor", "Key")
         else:
-            raise RuntimeError("Invalid data type")
+            raise TypeError("Invalid data type")
         self.title += ' ('+str(len(self.keys))+' '+ \
                       translate("DictEditor", "elements")+')'
         if self.remote:
@@ -274,6 +295,12 @@ class ReadOnlyDictModel(QAbstractTableModel):
         """Array row number"""
         return len(self.keys)
     
+    def get_index_from_key(self, key):
+        try:
+            return self.createIndex(self.keys.index(key), 0)
+        except ValueError:
+            return QModelIndex()
+    
     def get_key(self, index):
         """Return current key"""
         return self.keys[index.row()]
@@ -390,7 +417,8 @@ class DictModel(ReadOnlyDictModel):
             return False
         if index.column()<3:
             return False
-        value = display_to_value( value, self.get_value(index) )
+        value = display_to_value(value, self.get_value(index),
+                                 ignore_errors=True)
         self.set_value(index, value)
         self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
                   index, index)
@@ -402,6 +430,7 @@ class DictDelegate(QItemDelegate):
     def __init__(self, parent=None, inplace=False):
         QItemDelegate.__init__(self, parent)
         self.inplace = inplace
+        self._editors = {} # keep references on opened editors
         
     def get_value(self, index):
         if index.isValid():
@@ -415,15 +444,25 @@ class DictDelegate(QItemDelegate):
         """Overriding method createEditor"""
         if index.column()<3:
             return None
-        value = self.get_value(index)
+        try:
+            value = self.get_value(index)
+        except Exception, msg:
+            QMessageBox.critical(self.parent(),
+                                 translate("DictEditor", "Edit item"),
+                                 translate("DictEditor",
+                                           "<b>Unable to retrieve data.</b>"
+                                           "<br><br>Error message:<br>%1"
+                                           ).arg(unicode(msg)))
+            return
         key = index.model().get_key(index)
-        readonly = isinstance(value, tuple) or self.parent().readonly
+        readonly = isinstance(value, tuple) or self.parent().readonly \
+                   or not is_known_type(value)
         #---editor = DictEditor
         if isinstance(value, (list, tuple, dict)) and not self.inplace:
             editor = DictEditor(value, key, icon=self.parent().windowIcon(),
                                 readonly=readonly)
-            if editor.exec_() and not readonly:
-                self.set_value(index, editor.get_copy())
+            self.create_dialog(editor, dict(model=index.model(), editor=editor,
+                                            key=key, readonly=readonly))
             return None
         #---editor = ArrayEditor
         elif isinstance(value, ndarray) and ndarray is not FakeObject \
@@ -431,16 +470,24 @@ class DictDelegate(QItemDelegate):
             if value.size == 0:
                 return None
             editor = ArrayEditor(parent)
-            if editor.setup_and_check(value, title=key, readonly=readonly):
-                if editor.exec_():
-                    # Only necessary for child class RemoteDictDelegate:
-                    # (ArrayEditor does not make a copy of value)
-                    self.set_value(index, value)
+            if not editor.setup_and_check(value, title=key, readonly=readonly):
+                return
+            self.create_dialog(editor, dict(model=index.model(), editor=editor,
+                                            key=key, readonly=readonly))
             return None
         #---showing image
         elif isinstance(value, Image) and ndarray is not FakeObject \
              and Image is not FakeObject:
-            value.show()
+            arr = array(value)
+            if arr.size == 0:
+                return None
+            editor = ArrayEditor(parent)
+            if not editor.setup_and_check(arr, title=key, readonly=readonly):
+                return
+            conv_func = lambda arr: PIL.Image.fromarray(arr, mode=value.mode)
+            self.create_dialog(editor, dict(model=index.model(), editor=editor,
+                                            key=key, readonly=readonly,
+                                            conv=conv_func))
             return None
         #---editor = QDateTimeEdit
         elif isinstance(value, datetime.datetime) and not self.inplace:
@@ -461,9 +508,8 @@ class DictDelegate(QItemDelegate):
         #---editor = QTextEdit
         elif isinstance(value, (str, unicode)) and len(value)>40:
             editor = TextEditor(value, key)
-            if editor.exec_() and not readonly:
-                conv = str if isinstance(value, str) else unicode
-                self.set_value(index, conv(editor.get_copy()))
+            self.create_dialog(editor, dict(model=index.model(), editor=editor,
+                                            key=key, readonly=readonly))
             return None
         #---editor = QLineEdit
         else:
@@ -473,6 +519,26 @@ class DictDelegate(QItemDelegate):
             self.connect(editor, SIGNAL("returnPressed()"),
                          self.commitAndCloseEditor)
             return editor
+            
+    def create_dialog(self, editor, data):
+        self._editors[id(editor)] = data
+        self.connect(editor, SIGNAL('accepted()'),
+                     lambda eid=id(editor): self.editor_accepted(eid))
+        self.connect(editor, SIGNAL('rejected()'),
+                     lambda eid=id(editor): self.editor_rejected(eid))
+        editor.show()
+        
+    def editor_accepted(self, editor_id):
+        data = self._editors[editor_id]
+        if not data['readonly']:
+            index = data['model'].get_index_from_key(data['key'])
+            value = data['editor'].get_value()
+            conv_func = data.get('conv', lambda v: v)
+            self.set_value(index, conv_func(value))
+        self._editors.pop(editor_id)
+        
+    def editor_rejected(self, editor_id):
+        self._editors.pop(editor_id)
 
     def commitAndCloseEditor(self):
         """Overriding method commitAndCloseEditor"""
@@ -486,7 +552,7 @@ class DictDelegate(QItemDelegate):
         value = self.get_value(index)
         if isinstance(editor, QLineEdit):
             if not isinstance(value, basestring):
-                value = unicode(value)
+                value = repr(value)
             editor.setText(value)
         elif isinstance(editor, QDateEdit):
             editor.setDate(value)
@@ -516,7 +582,16 @@ class DictDelegate(QItemDelegate):
             # Should not happen...
             raise RuntimeError("Unsupported editor widget")
         
-        value = display_to_value( QVariant(value), self.get_value(index) )        
+        try:
+            value = display_to_value(QVariant(value), self.get_value(index),
+                                     ignore_errors=False)
+        except Exception, msg:
+            QMessageBox.critical(editor, translate("DictEditor", "Edit item"),
+                                 translate("DictEditor",
+                                       "<b>Unable to assign data to item.</b>"
+                                       "<br><br>Error message:<br>%1"
+                                       ).arg(str(msg)))
+            return
         self.set_value(index, value)
 
 
@@ -526,6 +601,21 @@ class BaseTableView(QTableView):
         self.array_filename = None
         self.menu = None
         self.empty_ws_menu = None
+        self.paste_action = None
+        self.copy_action = None
+        self.edit_action = None
+        self.plot_action = None
+        self.imshow_action = None
+        self.save_array_action = None
+        self.insert_action = None
+        self.remove_action = None
+        self.truncate_action = None
+        self.minmax_action = None
+        self.collvalue_action = None
+        self.inplace_action = None
+        self.rename_action = None
+        self.duplicate_action = None
+        self.delegate = None
         
     def setup_table(self):
         """Setup table"""
@@ -537,6 +627,13 @@ class BaseTableView(QTableView):
     
     def setup_menu(self, truncate, minmax, inplace, collvalue):
         """Setup context menu"""
+        if self.truncate_action is not None:
+            self.truncate_action.setChecked(truncate)
+            self.minmax_action.setChecked(minmax)
+            self.inplace_action.setChecked(inplace)
+            self.collvalue_action.setChecked(collvalue)
+            return
+        
         self.empty_ws_menu = QMenu(self)
         self.paste_action = create_action(self,
                                       translate("DictEditor", "Paste"),
@@ -619,6 +716,64 @@ class BaseTableView(QTableView):
         add_actions(menu, menu_actions)
         return menu
     
+    #------ Remote/local API ---------------------------------------------------
+    def remove_values(self, keys):
+        """Remove values from data"""
+        raise NotImplementedError
+
+    def copy_value(self, orig_key, new_key):
+        """Copy value"""
+        raise NotImplementedError
+    
+    def new_value(self, key, value):
+        """Create new value in data"""
+        raise NotImplementedError
+        
+    def is_list(self, key):
+        """Return True if variable is a list or a tuple"""
+        raise NotImplementedError
+        
+    def get_len(self, key):
+        """Return sequence length"""
+        raise NotImplementedError
+        
+    def is_array(self, key):
+        """Return True if variable is a numpy array"""
+        raise NotImplementedError
+
+    def is_image(self, key):
+        """Return True if variable is a PIL.Image image"""
+        raise NotImplementedError
+    
+    def is_dict(self, key):
+        """Return True if variable is a dictionary"""
+        raise NotImplementedError
+        
+    def get_array_shape(self, key):
+        """Return array's shape"""
+        raise NotImplementedError
+        
+    def get_array_ndim(self, key):
+        """Return array's ndim"""
+        raise NotImplementedError
+    
+    def oedit(self, key):
+        """Edit item"""
+        raise NotImplementedError
+    
+    def plot(self, key):
+        """Plot item"""
+        raise NotImplementedError
+    
+    def imshow(self, key):
+        """Show item's image"""
+        raise NotImplementedError
+    
+    def show_image(self, key):
+        """Show image (item is a PIL image)"""
+        raise NotImplementedError
+    #---------------------------------------------------------------------------
+            
     def refresh_menu(self):
         """Refresh context menu"""
         index = self.currentIndex()
@@ -629,13 +784,14 @@ class BaseTableView(QTableView):
         
     def refresh_plot_entries(self, index):
         if index.isValid():
-            value = self.delegate.get_value(index)
-            is_list = isinstance(value, (tuple, list))
-            is_array = isinstance(value, ndarray) and len(value) != 0
-            condition_plot = (is_array and len(value.shape) <= 2)
-            condition_imshow = condition_plot and value.ndim == 2
+            key = self.model.get_key(index)
+            is_list = self.is_list(key)
+            is_array = self.is_array(key) and self.get_len(key) != 0
+            condition_plot = (is_array and len(self.get_array_shape(key)) <= 2)
+            condition_imshow = condition_plot and self.get_array_ndim(key) == 2
+            condition_imshow = condition_imshow or self.is_image(key)
         else:
-            is_array = condition_plot = condition_imshow = False
+            is_array = condition_plot = condition_imshow = is_list = False
         self.plot_action.setVisible(condition_plot or is_list)
         self.imshow_action.setVisible(condition_imshow)
         self.save_array_action.setVisible(is_array)
@@ -667,6 +823,14 @@ class BaseTableView(QTableView):
             self.clearSelection()
             event.accept()
     
+    def mouseDoubleClickEvent(self, event):
+        """Reimplement Qt method"""
+        index_clicked = self.indexAt(event.pos())
+        if index_clicked.isValid():
+            self.edit_item()
+        else:
+            event.accept()
+    
     def keyPressEvent(self, event):
         """Reimplement Qt methods"""
         if event.key() == Qt.Key_Delete:
@@ -801,28 +965,28 @@ class BaseTableView(QTableView):
             
     def __prepare_plot(self):
         try:
-            from spyderlib import mpl_patch
-            mpl_patch.configure(backend="Qt4Agg")
-            mpl_patch.apply()
+            import guiqwt.pyplot #@UnusedImport
             return True
         except ImportError:
-            QMessageBox.warning(self, translate("DictEditor", "Import error"),
-                    translate("DictEditor",
-                              "Please install <b>matplotlib</b>."))
+            try:
+                if 'matplotlib' not in sys.modules:
+                    from spyderlib import mpl_patch
+                    mpl_patch.set_backend("Qt4Agg")
+                    mpl_patch.apply()
+                return True
+            except ImportError:
+                QMessageBox.warning(self, translate("DictEditor", "Import error"),
+                                    translate("DictEditor"
+                                              "Please install <b>matplotlib</b>"
+                                              " or <b>guiqwt</b>."))
 
     def plot_item(self):
         """Plot item"""
         index = self.currentIndex()
         if self.__prepare_plot():
+            key = self.model.get_key(index)
             try:
-                import guiqwt.pyplot as plt
-            except ImportError:
-                import matplotlib.pyplot as plt
-            data = self.delegate.get_value(index)
-            try:
-                plt.figure()
-                plt.plot(data)
-                plt.show()
+                self.plot(key)
             except ValueError, error:
                 QMessageBox.critical(self, translate("DictEditor", "Plot"),
                     translate("DictEditor", "<b>Unable to plot data.</b>"
@@ -832,15 +996,12 @@ class BaseTableView(QTableView):
         """Imshow item"""
         index = self.currentIndex()
         if self.__prepare_plot():
+            key = self.model.get_key(index)
             try:
-                import guiqwt.pyplot as plt
-            except ImportError:
-                import matplotlib.pyplot as plt
-            data = self.delegate.get_value(index)
-            try:
-                plt.figure()
-                plt.imshow(data)
-                plt.show()
+                if self.is_image(key):
+                    self.show_image(key)
+                else:
+                    self.imshow(key)
             except ValueError, error:
                 QMessageBox.critical(self, translate("DictEditor", "Plot"),
                     translate("DictEditor", "<b>Unable to show image.</b>"
@@ -870,7 +1031,7 @@ class BaseTableView(QTableView):
         clipboard = QApplication.clipboard()
         clipl = []
         for idx in self.selectedIndexes():
-            if not idx.isValid:
+            if not idx.isValid():
                 continue
             clipl.append(unicode(self.delegate.get_value(idx)))
         clipboard.setText(u'\n'.join(clipl))
@@ -912,8 +1073,6 @@ class DictEditorTableView(BaseTableView):
         BaseTableView.__init__(self, parent)
         self.dictfilter = None
         self.readonly = readonly or isinstance(data, tuple)
-        self.model = None
-        self.delegate = None
         DictModelClass = ReadOnlyDictModel if self.readonly else DictModel
         self.model = DictModelClass(self, data, title, names=names,
                                     truncate=truncate, minmax=minmax,
@@ -925,34 +1084,89 @@ class DictEditorTableView(BaseTableView):
         self.setup_table()
         self.menu = self.setup_menu(truncate, minmax, inplace, collvalue)
     
+    #------ Remote/local API ---------------------------------------------------
     def remove_values(self, keys):
-        """
-        Remove values from data
-        (implemented differently for remote table view)
-        """
+        """Remove values from data"""
         data = self.model.get_data()
         for key in sorted(keys,reverse=True):
             data.pop(key)
             self.set_data(data)
 
     def copy_value(self, orig_key, new_key):
-        """
-        Copy value
-        (implemented differently for remote table view)
-        """
+        """Copy value"""
         data = self.model.get_data()
         data[new_key] = data[orig_key]
         self.set_data(data)
     
     def new_value(self, key, value):
-        """
-        Create new value in data
-        (implemented differently for remote table view)
-        """
+        """Create new value in data"""
         data = self.model.get_data()
         data[key] = value
         self.set_data(data)
+        
+    def is_list(self, key):
+        """Return True if variable is a list or a tuple"""
+        data = self.model.get_data()
+        return isinstance(data[key], (tuple, list))
+        
+    def get_len(self, key):
+        """Return sequence length"""
+        data = self.model.get_data()
+        return len(data[key])
+        
+    def is_array(self, key):
+        """Return True if variable is a numpy array"""
+        data = self.model.get_data()
+        return isinstance(data[key], ndarray)
+        
+    def is_image(self, key):
+        """Return True if variable is a PIL.Image image"""
+        data = self.model.get_data()
+        return isinstance(data[key], Image)
+    
+    def is_dict(self, key):
+        """Return True if variable is a dictionary"""
+        data = self.model.get_data()
+        return isinstance(data[key], dict)
+        
+    def get_array_shape(self, key):
+        """Return array's shape"""
+        data = self.model.get_data()
+        return data[key].shape
+        
+    def get_array_ndim(self, key):
+        """Return array's ndim"""
+        data = self.model.get_data()
+        return data[key].ndim
+    
+    def oedit(self, key):
+        """Edit item"""
+        data = self.model.get_data()
+        from spyderlib.widgets.objecteditor import oedit
+        oedit(data[key])
+    
+    def plot(self, key):
+        """Plot item"""
+        data = self.model.get_data()
+        import spyderlib.pyplot as plt
+        plt.figure()
+        plt.plot(data[key])
+        plt.show()
+    
+    def imshow(self, key):
+        """Show item's image"""
+        data = self.model.get_data()
+        import spyderlib.pyplot as plt
+        plt.figure()
+        plt.imshow(data[key])
+        plt.show()
             
+    def show_image(self, key):
+        """Show image (item is a PIL image)"""
+        data = self.model.get_data()
+        data[key].show()
+    #---------------------------------------------------------------------------
+        
     def refresh_menu(self):
         """Refresh context menu"""
         data = self.model.get_data()
@@ -1031,7 +1245,7 @@ class DictEditor(QDialog):
         # Make the dialog act as a window
         self.setWindowFlags(Qt.Window)
         
-    def get_copy(self):
+    def get_value(self):
         """Return modified copy of dictionary or list"""
         return self.data_copy
     
@@ -1046,10 +1260,12 @@ def dedit(seq):
     (instantiate a new QApplication if necessary,
     so it can be called directly from the interpreter)
     """
-    _app = qapplication()
+    app = qapplication()
     dialog = DictEditor(seq)
-    if dialog.exec_():
-        return dialog.get_copy()
+    dialog.show()
+    app.exec_()
+    if dialog.result():
+        return dialog.get_value()
 
 
 #----Remote versions of DictDelegate and DictEditorTableView
@@ -1062,26 +1278,46 @@ class RemoteDictDelegate(DictDelegate):
         self.set_value_func = set_value_func
         
     def get_value(self, index):
-        name = index.model().keys[index.row()]
-        return self.get_value_func(name)
+        if index.isValid():
+            name = index.model().keys[index.row()]
+            return self.get_value_func(name)
     
     def set_value(self, index, value):
-        name = index.model().keys[index.row()]
-        self.set_value_func(name, value)
+        if index.isValid():
+            name = index.model().keys[index.row()]
+            self.set_value_func(name, value)
         
 class RemoteDictEditorTableView(BaseTableView):
     """DictEditor table view"""
-    def __init__(self, parent, data,
-                 truncate=True, minmax=False, inplace=False, collvalue=True,
+    def __init__(self, parent, data, truncate=True, minmax=False,
+                 inplace=False, collvalue=True, remote_editing=False,
                  get_value_func=None, set_value_func=None,
                  new_value_func=None, remove_values_func=None,
-                 copy_value_func=None):
+                 copy_value_func=None, is_list_func=None, get_len_func=None,
+                 is_array_func=None, is_image_func=None, is_dict_func=None,
+                 get_array_shape_func=None, get_array_ndim_func=None,
+                 oedit_func=None, plot_func=None, imshow_func=None,
+                 show_image_func=None):
         BaseTableView.__init__(self, parent)
         
+        self.remote_editing_enabled = None
+        
         self.remove_values = remove_values_func
         self.copy_value = copy_value_func
         self.new_value = new_value_func
         
+        self.is_list = is_list_func
+        self.get_len = get_len_func
+        self.is_array = is_array_func
+        self.is_image = is_image_func
+        self.is_dict = is_dict_func
+        self.get_array_shape = get_array_shape_func
+        self.get_array_ndim = get_array_ndim_func
+        self.oedit = oedit_func
+        self.plot = plot_func
+        self.imshow = imshow_func
+        self.show_image = show_image_func
+        
         self.dictfilter = None
         self.model = None
         self.delegate = None
@@ -1095,7 +1331,57 @@ class RemoteDictEditorTableView(BaseTableView):
         self.setItemDelegate(self.delegate)
         
         self.setup_table()
-        self.menu = self.setup_menu(truncate, minmax, inplace, collvalue)
+        self.menu = self.setup_menu(truncate, minmax, inplace, collvalue,
+                                    remote_editing)
+
+    def setup_menu(self, truncate, minmax, inplace, collvalue, remote_editing):
+        """Setup context menu"""
+        menu = BaseTableView.setup_menu(self, truncate, minmax,
+                                        inplace, collvalue)
+        if menu is None:
+            self.remote_editing_action.setChecked(remote_editing)
+            return
+        
+        self.remote_editing_action = create_action(self,
+                translate("DictEditor", "Edit data in the remote process"),
+                tip=translate("DictEditor",
+                      "Editors are opened in the remote process for NumPy "
+                      "arrays, PIL images, lists, tuples and dictionaries.\n"
+                      "This avoids transfering large amount of data between "
+                      "the remote process and Spyder (through the socket)."),
+                toggled=self.toggle_remote_editing)
+        self.remote_editing_action.setChecked(remote_editing)
+        self.toggle_remote_editing(remote_editing)
+        add_actions(menu, (self.remote_editing_action,))
+        return menu
+            
+    def toggle_remote_editing(self, state):
+        """Toggle remote editing state"""
+        self.emit(SIGNAL('option_changed'), 'remote_editing', state)
+        self.remote_editing_enabled = state
+            
+    def edit_item(self):
+        """
+        Reimplement BaseTableView's method to edit item
+        
+        Some supported data types are directly edited in the remote process,
+        thus avoiding to transfer large amount of data through the socket from
+        the remote process to Spyder
+        """
+        if self.remote_editing_enabled:
+            index = self.currentIndex()
+            if not index.isValid():
+                return
+            key = self.model.get_key(index)
+            if (self.is_list(key) or self.is_dict(key) 
+                or self.is_array(key) or self.is_image(key)):
+                # If this is a remote dict editor, the following avoid 
+                # transfering large amount of data through the socket
+                self.oedit(key)
+            else:
+                BaseTableView.edit_item(self)
+        else:
+            BaseTableView.edit_item(self)
 
 
 #----Globals filter: filter namespace dictionaries (to be edited in DictEditor)
@@ -1121,27 +1407,27 @@ def globalsfilter(input_dict, itermax=-1, filters=None,
                   exclude_private=None, exclude_upper=None,
                   exclude_unsupported=None, excluded_names=None):
     """Keep only objects that can be pickled"""
-    output_dict = input_dict.copy() # Shallow copy
-    for key in input_dict:
-        if (exclude_private and key.startswith('_')) or \
-           (exclude_upper and key[0].isupper()) or \
-           (key in excluded_names) or \
-           (exclude_unsupported and not is_supported(input_dict[key],
-                                                     itermax=itermax,
-                                                     filters=filters)):
-            output_dict.pop(key)
+    output_dict = {}
+    for key, value in input_dict.items():
+        excluded = (exclude_private and key.startswith('_')) or \
+                   (exclude_upper and key[0].isupper()) or \
+                   (key in excluded_names) or \
+                   (exclude_unsupported and \
+                    not is_supported(value, itermax=itermax, filters=filters))
+        if not excluded:
+            output_dict[key] = value
     return output_dict
 
 
 def get_test_data():
     """Create test data"""
-    import numpy as np, PIL.Image
-    image = PIL.Image.fromarray(np.random.rand(100, 100))
+    import numpy as np
+    image = PIL.Image.fromarray(np.random.random_integers(255, size=(100, 100)))
     testdict = {'d': 1, 'a': np.random.rand(10, 10), 'b': [1, 2]}
     testdate = datetime.date(1945, 5, 8)
     return {'str': 'kjkj kj k j j kj k jkj',
             'unicode': u'éù',
-            'list': [1, 3, [4, 5, 6], 'kjkj', None],
+            'list': [1, 3, [sorted, 5, 6], 'kjkj', None],
             'tuple': ([1, testdate, testdict], 'kjkj', None),
             'dict': testdict,
             'float': 1.2233,
@@ -1151,6 +1437,8 @@ def get_test_data():
             'image': image,
             'date': testdate,
             'datetime': datetime.datetime(1945, 5, 8),
+            'unsupported1': np.arccos,
+            'unsupported2': np.cast,
             }
 
 def test():
@@ -1160,15 +1448,17 @@ def test():
     
 def remote_editor_test():
     """Remote dictionary editor test"""
-    from spyderlib.widgets.externalshell.globalsexplorer import get_settings
+    from spyderlib.plugins.variableexplorer import VariableExplorer
     from spyderlib.widgets.externalshell.monitor import make_remote_view
-    remote = make_remote_view(get_test_data(), get_settings())
+    remote = make_remote_view(get_test_data(), VariableExplorer.get_settings())
     from pprint import pprint
     pprint(remote)
-    _app = qapplication()
+    app = qapplication()
     dialog = DictEditor(remote, remote=True)
-    if dialog.exec_():
-        print dialog.get_copy()
+    dialog.show()
+    app.exec_()
+    if dialog.result():
+        print dialog.get_value()
 
 if __name__ == "__main__":
-    test()
+    test()
\ No newline at end of file
