@@ -4,7 +4,7 @@
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
-"""Editor tools: class browser, etc."""
+"""Editor tools: outline explorer, etc."""
 
 import sys, re,  os.path as osp
 
@@ -19,7 +19,7 @@ STDOUT = sys.stdout
 from spyderlib.config import get_icon
 from spyderlib.utils.qthelpers import (create_action, translate,
                                        create_toolbutton, set_item_user_text)
-from spyderlib.widgets import OneColumnTree
+from spyderlib.widgets.onecolumntree import OneColumnTree
 
 
 #===============================================================================
@@ -27,13 +27,14 @@ from spyderlib.widgets import OneColumnTree
 #===============================================================================
 import compiler
 
-def check(filename):
+def check(source_code, filename=None):
     try:
-        import pyflakes.checker
+        import pyflakes.checker, pyflakes.messages
     except ImportError:
         return []
     try:
-        tree = compiler.parse(file(filename, 'U').read() + '\n')
+#        tree = compiler.parse(file(filename, 'U').read() + '\n')
+        tree = compiler.parse(source_code+'\n')
     except (SyntaxError, IndentationError), e:
         message = e.args[0]
         value = sys.exc_info()[1]
@@ -45,11 +46,20 @@ def check(filename):
         return [ (message, lineno, True) ]
     else:
         results = []
+        if filename is None:
+            filename = '(none)'
         w = pyflakes.checker.Checker(tree, filename)
         w.messages.sort(lambda a, b: cmp(a.lineno, b.lineno))
+        
+        error_messages = (pyflakes.messages.UndefinedName,
+                          pyflakes.messages.UndefinedExport,
+                          pyflakes.messages.UndefinedLocal,
+                          pyflakes.messages.DuplicateArgument,
+                          pyflakes.messages.LateFutureImport)
         for warning in w.messages:
             results.append( (warning.message % warning.message_args,
-                             warning.lineno, False) )
+                             warning.lineno,
+                             isinstance(warning, error_messages)) )
         return results
 
 if __name__ == '__main__':
@@ -65,8 +75,8 @@ class PythonCFM(object):
     """
     Collection of helpers to match functions and classes
     for Python language
-    This has to be reimplemented for other languages for the class browser 
-    to be supported (not implemented yet: class browser won't be populated
+    This has to be reimplemented for other languages for the outline explorer 
+    to be supported (not implemented yet: outline explorer won't be populated
     unless the current script is a Python script)
     """
     def __get_name(self, statmt, text):
@@ -112,8 +122,13 @@ class TreeItem(QTreeWidgetItem):
                 # or have the same parent as item
                 while preceding.parent() is not parent:
                     preceding = preceding.parent()
-            QTreeWidgetItem.__init__(self, parent, preceding,
-                                     QTreeWidgetItem.Type)
+                    if preceding is None:
+                        break
+            if preceding is None:
+                QTreeWidgetItem.__init__(self, parent, QTreeWidgetItem.Type)
+            else:
+                QTreeWidgetItem.__init__(self, parent, preceding,
+                                         QTreeWidgetItem.Type)
         self.setText(0, name)
         parent_text = unicode(parent.data(0, Qt.UserRole).toString())
         set_item_user_text(self, parent_text+'/'+name)
@@ -123,17 +138,18 @@ class TreeItem(QTreeWidgetItem):
         self.setIcon(0, get_icon(icon_name))
         
     def setup(self):
-        raise NotImplementedError
+        self.setToolTip(0, translate("OutlineExplorer",
+                                     "Line %1").arg(str(self.line)))
 
 class ClassItem(TreeItem):
     def setup(self):
         self.set_icon('class.png')
-        self.setToolTip(0, translate("ClassBrowser",
+        self.setToolTip(0, translate("OutlineExplorer",
                              "Class defined at line %1").arg(str(self.line)))
 
 class FunctionItem(TreeItem):
     def __init__(self, name, line, parent, preceding):
-        super(FunctionItem, self).__init__(name, line, parent, preceding)
+        TreeItem.__init__(self, name, line, parent, preceding)
         self.decorator = None
         
     def set_decorator(self, decorator):
@@ -144,7 +160,7 @@ class FunctionItem(TreeItem):
     
     def setup(self):
         if self.is_method():
-            self.setToolTip(0, translate("ClassBrowser",
+            self.setToolTip(0, translate("OutlineExplorer",
                              "Method defined at line %1").arg(str(self.line)))
             if self.decorator is not None:
                 self.set_icon('decorator.png')
@@ -158,7 +174,7 @@ class FunctionItem(TreeItem):
                     self.set_icon('method.png')
         else:
             self.set_icon('function.png')
-            self.setToolTip(0, translate("ClassBrowser",
+            self.setToolTip(0, translate("OutlineExplorer",
                              "Function defined at line %1").arg(str(self.line)))
 
 
@@ -193,37 +209,45 @@ def remove_from_tree_cache(tree_cache, line=None, item=None):
         #XXX: remove this debug-related fragment of code
         print >>STDOUT, "unable to remove tree item: ", debug
 
-class ClassBrowserTreeWidget(OneColumnTree):
+class OutlineExplorerTreeWidget(OneColumnTree):
     def __init__(self, parent, show_fullpath=False, fullpath_sorting=True,
-                 show_all_files=True):
+                 show_all_files=True, show_comments=True):
         self.show_fullpath = show_fullpath
         self.fullpath_sorting = fullpath_sorting
         self.show_all_files = show_all_files
+        self.show_comments = show_comments
         OneColumnTree.__init__(self, parent)
         self.freeze = False # Freezing widget to avoid any unwanted update
         self.editor_items = {}
         self.editor_tree_cache = {}
         self.editor_ids = {}
         self.current_editor = None
-        title = translate("ClassBrowser", "Classes and functions")
+        title = translate("OutlineExplorer", "Outline")
         self.set_title(title)
         self.setWindowTitle(title)
+        self.setUniformRowHeights(True)
 
     def get_actions_from_items(self, items):
         """Reimplemented OneColumnTree method"""
         fromcursor_act = create_action(self,
-                        text=translate('ClassBrowser', 'Go to cursor position'),
+                        text=translate('OutlineExplorer',
+                                       'Go to cursor position'),
                         icon=get_icon('fromcursor.png'),
                         triggered=self.go_to_cursor_position)
         fullpath_act = create_action(self,
-                        text=translate('ClassBrowser', 'Show absolute path'),
+                        text=translate('OutlineExplorer', 'Show absolute path'),
                         toggled=self.toggle_fullpath_mode)
         fullpath_act.setChecked(self.show_fullpath)
         allfiles_act = create_action(self,
-                        text=translate('ClassBrowser', 'Show all files'),
+                        text=translate('OutlineExplorer', 'Show all files'),
                         toggled=self.toggle_show_all_files)
         allfiles_act.setChecked(self.show_all_files)
-        actions = [fullpath_act, allfiles_act, fromcursor_act]
+        comment_act = create_action(self,
+                        text=translate('OutlineExplorer',
+                                       'Show special comments'),
+                        toggled=self.toggle_show_comments)
+        comment_act.setChecked(self.show_comments)
+        actions = [fullpath_act, allfiles_act, comment_act, fromcursor_act]
         return actions
     
     def toggle_fullpath_mode(self, state):
@@ -247,6 +271,10 @@ class ClassBrowserTreeWidget(OneColumnTree):
             item = self.editor_items[editor_id]
             self.__hide_or_show_root_items(item)
             
+    def toggle_show_comments(self, state):
+        self.show_comments = state
+        self.update_all()
+            
     def set_fullpath_sorting(self, state):
         self.fullpath_sorting = state
         self.__sort_toplevel_items()
@@ -296,11 +324,12 @@ class ClassBrowserTreeWidget(OneColumnTree):
         self.current_editor = editor
         
     def file_renamed(self, editor, new_filename):
-        """File was renamed, updating class browser tree"""
+        """File was renamed, updating outline explorer tree"""
         editor_id = editor.get_document_id()
         if editor_id in self.editor_ids.values():
             root_item = self.editor_items[editor_id]
             root_item.set_path(new_filename, fullpath=self.show_fullpath)
+            self.__sort_toplevel_items()
         
     def update_all(self):
         self.save_expanded_state()
@@ -347,38 +376,16 @@ class ClassBrowserTreeWidget(OneColumnTree):
         previous_item = None
         previous_level = None
         
-        if editor.highlighter is not None:
-            # QtEditor
-            cb_data = editor.highlighter.get_classbrowser_data()
-        else:
-            # QsciEditor
-            cb_data = None
+        oe_data = editor.highlighter.get_outlineexplorer_data()
+        
         for block_nb in range(editor.get_line_count()):
             line_nb = block_nb+1
-            if cb_data is not None:
-                data = cb_data.get(block_nb)
-            else:
-                from spyderlib.widgets.qteditor.syntaxhighlighters import ClassBrowserData
-                data = ClassBrowserData()
-                text = unicode(editor.text(block_nb))
-                data.text = text
-                data.fold_level = editor.get_fold_level(block_nb)
-                class_name = editor.classfunc_match.get_class_name(text)
-                if class_name is not None:
-                    data.def_type = ClassBrowserData.CLASS
-                    data.def_name = class_name
-                else:
-                    func_name = editor.classfunc_match.get_function_name(text)
-                    if func_name is not None:
-                        data.def_type = ClassBrowserData.FUNCTION
-                        data.def_name = func_name
-            
+            data = oe_data.get(block_nb)
             if data is None:
                 level = None
             else:
                 level = data.fold_level
-            citem, clevel, _d = tree_cache.get(line_nb,
-                                                      (None, None, ""))
+            citem, clevel, _d = tree_cache.get(line_nb, (None, None, ""))
             
             # Skip iteration if line is not the first line of a foldable block
             if level is None:
@@ -387,17 +394,21 @@ class ClassBrowserTreeWidget(OneColumnTree):
                 continue
             
             # Searching for class/function statements
-            class_name = data.get_class_name()
-            if class_name is None:
-                func_name = data.get_function_name()
-                if func_name is None:
-                    if citem is not None:
-                        remove_from_tree_cache(tree_cache, line=line_nb)
-                    continue
+            not_class_nor_function = data.is_not_class_nor_function()
+            if not not_class_nor_function:
+                class_name = data.get_class_name()
+                if class_name is None:
+                    func_name = data.get_function_name()
+                    if func_name is None:
+                        if citem is not None:
+                            remove_from_tree_cache(tree_cache, line=line_nb)
+                        continue
                 
             if previous_level is not None:
                 if level == previous_level:
                     pass
+                elif level > previous_level+4: # Invalid indentation
+                    continue
                 elif level > previous_level:
                     ancestors.append((previous_item, previous_level))
                 else:
@@ -410,7 +421,20 @@ class ClassBrowserTreeWidget(OneColumnTree):
                 cname = unicode(citem.text(0))
                 
             preceding = root_item if previous_item is None else previous_item
-            if class_name is not None:
+            if not_class_nor_function:
+                if data.is_comment() and not self.show_comments:
+                    if citem is not None:
+                        remove_from_tree_cache(tree_cache, line=line_nb)
+                    continue
+                if citem is not None:
+                    if data.text == cname and level == clevel:
+                        previous_level = clevel
+                        previous_item = citem
+                        continue
+                    else:
+                        remove_from_tree_cache(tree_cache, line=line_nb)
+                item = TreeItem(data.text, line_nb, parent, preceding)
+            elif class_name is not None:
                 if citem is not None:
                     if class_name == cname and level == clevel:
                         previous_level = clevel
@@ -429,9 +453,10 @@ class ClassBrowserTreeWidget(OneColumnTree):
                         remove_from_tree_cache(tree_cache, line=line_nb)
                 item = FunctionItem(func_name, line_nb, parent, preceding)
                 if item.is_method() and line_nb > 1:
-                    text = unicode(editor.text(block_nb-1))
+                    text = editor.get_text_line(block_nb-1)
                     decorator = editor.classfunc_match.get_decorator(text)
                     item.set_decorator(decorator)
+                
             item.setup()
             debug = "%s -- %s/%s" % (str(item.line).rjust(6),
                                      unicode(item.parent().text(0)),
@@ -491,7 +516,7 @@ class ClassBrowserTreeWidget(OneColumnTree):
                 break
     
     
-class ClassBrowser(QWidget):
+class OutlineExplorer(QWidget):
     """
     Class browser
     
@@ -499,17 +524,18 @@ class ClassBrowser(QWidget):
         SIGNAL("edit_goto(QString,int,QString)")
     """
     def __init__(self, parent=None, show_fullpath=True, fullpath_sorting=True,
-                 show_all_files=True):
+                 show_all_files=True, show_comments=True):
         QWidget.__init__(self, parent)
 
-        self.treewidget = ClassBrowserTreeWidget(self,
+        self.treewidget = OutlineExplorerTreeWidget(self,
                                             show_fullpath=show_fullpath,
                                             fullpath_sorting=fullpath_sorting,
-                                            show_all_files=show_all_files)
+                                            show_all_files=show_all_files,
+                                            show_comments=show_comments)
 
         self.visibility_action = create_action(self,
-                                           self.tr("Show/hide class browser"),
-                                           icon='class_browser_vis.png',
+                                           self.tr("Show/hide outline explorer"),
+                                           icon='outline_explorer_vis.png',
                                            toggled=self.toggle_visibility)
         self.visibility_action.setChecked(True)
         
@@ -519,6 +545,7 @@ class ClassBrowser(QWidget):
             btn_layout.addWidget(btn)
 
         layout = QVBoxLayout()
+        layout.setContentsMargins(0, 0, 0, 0)
         layout.addWidget(self.treewidget)
         layout.addLayout(btn_layout)
         self.setLayout(layout)
@@ -530,33 +557,37 @@ class ClassBrowser(QWidget):
             current_editor.clearFocus()
             current_editor.setFocus()
             if state:
-                self.emit(SIGNAL("classbrowser_is_visible()"))
+                self.emit(SIGNAL("outlineexplorer_is_visible()"))
         
     def setup_buttons(self):
-        fromcursor_btn = create_toolbutton(self, get_icon("fromcursor.png"),
-                         tip=translate('ClassBrowser', 'Go to cursor position'),
-                         triggered=self.treewidget.go_to_cursor_position)
-        collapse_btn = create_toolbutton(self, text_beside_icon=False)
+        fromcursor_btn = create_toolbutton(self,
+                             icon=get_icon("fromcursor.png"),
+                             tip=translate('OutlineExplorer',
+                                           'Go to cursor position'),
+                             triggered=self.treewidget.go_to_cursor_position)
+        collapse_btn = create_toolbutton(self)
         collapse_btn.setDefaultAction(self.treewidget.collapse_selection_action)
-        expand_btn = create_toolbutton(self, text_beside_icon=False)
+        expand_btn = create_toolbutton(self)
         expand_btn.setDefaultAction(self.treewidget.expand_selection_action)
-        restore_btn = create_toolbutton(self, text_beside_icon=False)
+        restore_btn = create_toolbutton(self)
         restore_btn.setDefaultAction(self.treewidget.restore_action)
         return (fromcursor_btn, collapse_btn, expand_btn, restore_btn)
         
     def set_current_editor(self, editor, fname, update):
-        self.treewidget.set_current_editor(editor, fname, update)
+        if editor.highlighter is not None:
+            self.treewidget.set_current_editor(editor, fname, update)
         
     def remove_editor(self, editor):
         self.treewidget.remove_editor(editor)
         
     def get_options(self):
         """
-        Return class browser options
+        Return outline explorer options
         except for fullpath sorting option which is more global
         """
         return dict(show_fullpath=self.treewidget.show_fullpath,
                     show_all_files=self.treewidget.show_all_files,
+                    show_comments=self.treewidget.show_comments,
                     expanded_state=self.treewidget.get_expanded_state(),
                     scrollbar_position=self.treewidget.get_scrollbar_position(),
                     visibility=self.isVisible())
@@ -576,22 +607,26 @@ class ClassBrowser(QWidget):
 #===============================================================================
 class LineNumberArea(QWidget):
     def __init__(self, editor):
-        super(LineNumberArea, self).__init__(editor)
+        QWidget.__init__(self, editor)
         self.code_editor = editor
         
     def sizeHint(self):
-        return QSize(self.code_editor.get_linenumberarea_width(), 0)
+        return QSize(self.code_editor.compute_linenumberarea_width(), 0)
         
     def paintEvent(self, event):
         self.code_editor.linenumberarea_paint_event(event)
 
     def mousePressEvent(self, event):
         self.code_editor.linenumberarea_mousepress_event(event)
+        
+    def mouseDoubleClickEvent(self, event):
+        self.code_editor.linenumberarea_mousedoubleclick_event(event)
 
 class ScrollFlagArea(QWidget):
     WIDTH = 12
     def __init__(self, editor):
-        super(ScrollFlagArea, self).__init__(editor)
+        QWidget.__init__(self, editor)
+        self.setAttribute(Qt.WA_OpaquePaintEvent)
         self.code_editor = editor
         
     def sizeHint(self):
@@ -609,7 +644,7 @@ class ScrollFlagArea(QWidget):
 
 class EdgeLine(QWidget):
     def __init__(self, editor):
-        super(EdgeLine, self).__init__(editor)
+        QWidget.__init__(self, editor)
         self.code_editor = editor
         self.column = 80
         
