@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -37,7 +37,7 @@ STDOUT = sys.stdout
 from spyderlib.widgets.formlayout import fedit
 from spyderlib.utils.qthelpers import (get_std_icon, create_action, add_actions,
                                        translate)
-from spyderlib.utils import encoding, rename_file, remove_file
+from spyderlib.utils import encoding, rename_file, remove_file, programs
 from spyderlib.config import get_icon
 
 
@@ -120,7 +120,7 @@ class DirView(QTreeView):
         return index
         
     def set_folder(self, folder, force_current=False):
-        if not force_current or self.is_in_current_folder(folder):
+        if not force_current:
             return
         index = self.refresh_folder(folder)
         self.expand(index)
@@ -140,6 +140,8 @@ class DirView(QTreeView):
 class ExplorerTreeWidget(DirView):
     def __init__(self, parent=None):
         DirView.__init__(self, parent)
+        self.history = []
+        self.histindex = None
         
     def setup(self, path=None, name_filters=['*.py', '*.pyw'],
               valid_types= ('.py', '.pyw'), show_all=False):
@@ -147,7 +149,9 @@ class ExplorerTreeWidget(DirView):
         self.valid_types = valid_types
         self.show_all = show_all
         
-        self.refresh(path)
+        if path is None:
+            path = os.getcwdu()
+        self.chdir(path)
         
         # Enable drag events
         self.setDragEnabled(True)
@@ -237,7 +241,7 @@ class ExplorerTreeWidget(DirView):
             open_action = create_action(self,
                                         translate('Explorer', "Open"),
                                         triggered=self.startfile)
-            if ext in ('.py', '.pyw'):
+            if ext in ('.py', '.pyw', '.ipy'):
                 actions.append(run_action)
             if ext in self.valid_types or os.name != 'nt':
                 actions.append(browse_action if is_dir else edit_action)
@@ -266,6 +270,13 @@ class ExplorerTreeWidget(DirView):
                                    self.parent_widget.emit(
                                    SIGNAL("open_interpreter(QString)"), _fn))
                 actions.append(action)
+                if programs.is_module_installed("IPython"):
+                    _title = translate('Explorer', "Open IPython here")
+                    action = create_action(self, _title, icon="ipython.png",
+                                       triggered=lambda _fn=fname:
+                                       self.parent_widget.emit(
+                                       SIGNAL("open_ipython(QString)"), _fn))
+                    actions.append(action)
         if actions:
             actions.append(None)
         actions += self.common_actions
@@ -281,7 +292,12 @@ class ExplorerTreeWidget(DirView):
         if new_path is None:
             new_path = os.getcwdu()
         self.set_folder(new_path, force_current=force_current)
-        
+        self.emit(SIGNAL("set_previous_enabled(bool)"),
+                  self.histindex is not None and self.histindex > 0)
+        self.emit(SIGNAL("set_next_enabled(bool)"),
+                  self.histindex is not None and \
+                  self.histindex < len(self.history)-1)
+    
         
     #---- Events
     def contextMenuEvent(self, event):
@@ -351,11 +367,50 @@ class ExplorerTreeWidget(DirView):
         fname = self.get_filename()
         if fname:
             if osp.isdir(fname):
-                self.parent_widget.emit(SIGNAL("open_dir(QString)"), fname)
-                self.refresh()
+                self.chdir(directory=unicode(fname))
             else:
                 self.open(fname)
         
+    def go_to_parent_directory(self):
+        self.chdir( osp.abspath(osp.join(os.getcwdu(), os.pardir)) )
+        
+    def go_to_previous_directory(self):
+        """Back to previous directory"""
+        self.histindex -= 1
+        self.chdir(browsing_history=True)
+        
+    def go_to_next_directory(self):
+        """Return to next directory"""
+        self.histindex += 1
+        self.chdir(browsing_history=True)
+        
+    def update_history(self, directory):
+        directory = osp.abspath(unicode(directory))
+        if directory in self.history:
+            self.histindex = self.history.index(directory)
+        
+    def chdir(self, directory=None, browsing_history=False):
+        """Set directory as working directory"""
+        if directory is not None:
+            directory = osp.abspath(unicode(directory))
+        if browsing_history:
+            directory = self.history[self.histindex]
+        elif directory in self.history:
+            self.histindex = self.history.index(directory)
+        else:
+            if self.histindex is None:
+                self.history = []
+            else:
+                self.history = self.history[:self.histindex+1]
+            if len(self.history) == 0 or \
+               (self.history and self.history[-1] != directory):
+                self.history.append(directory)
+            self.histindex = len(self.history)-1
+        directory = unicode(directory)
+        os.chdir(directory)
+        self.parent_widget.emit(SIGNAL("open_dir(QString)"), directory)
+        self.refresh(new_path=directory, force_current=True)
+        
     def open(self, fname):
         """Open filename with the appropriate application"""
         fname = unicode(fname)
@@ -464,14 +519,14 @@ class ExplorerTreeWidget(DirView):
         _temp = sys.stdout
         sys.stdout = None
         fname = QFileDialog.getSaveFileName(self,
-                translate('Explorer', "New Python script"), self.get_dirname(),
-                translate('Explorer', "Python scripts")+" (*.py ; *.pyw)"+"\n"+\
-                translate('Explorer', "All files")+" (*.*)")
+            translate('Explorer', "New Python script"), self.get_dirname(),
+            translate('Explorer', "Python scripts")+" (*.py ; *.pyw ; *.ipy)"+\
+            "\n"+translate('Explorer', "All files")+" (*.*)")
         sys.stdout = _temp
         if not fname.isEmpty():
             fname = unicode(fname)
             try:
-                if osp.splitext(fname)[1] in ('.py', '.pyw'):
+                if osp.splitext(fname)[1] in ('.py', '.pyw', '.ipy'):
                     create_script(fname)
                 else:
                     file(fname, 'wb').write('')
@@ -514,23 +569,27 @@ class ExplorerWidget(QWidget):
         self.toolbar.setIconSize(QSize(16, 16))
         
         self.previous_action = create_action(self,
-                    text=translate('Explorer', "Previous"),
-                    icon=get_icon('previous.png'),
-                    triggered=lambda: self.emit(SIGNAL("open_previous_dir()")))
+                            text=translate('Explorer', "Previous"),
+                            icon=get_icon('previous.png'),
+                            triggered=self.treewidget.go_to_previous_directory)
         self.toolbar.addAction(self.previous_action)
         self.previous_action.setEnabled(False)
+        self.connect(self.treewidget, SIGNAL("set_previous_enabled(bool)"),
+                     self.previous_action.setEnabled)
         
         self.next_action = create_action(self,
-                    text=translate('Explorer', "Next"),
-                    icon=get_icon('next.png'),
-                    triggered=lambda: self.emit(SIGNAL("open_next_dir()")))
+                            text=translate('Explorer', "Next"),
+                            icon=get_icon('next.png'),
+                            triggered=self.treewidget.go_to_next_directory)
         self.toolbar.addAction(self.next_action)
         self.next_action.setEnabled(False)
+        self.connect(self.treewidget, SIGNAL("set_next_enabled(bool)"),
+                     self.next_action.setEnabled)
         
         parent_action = create_action(self,
-                    text=translate('Explorer', "Parent"),
-                    icon=get_icon('up.png'),
-                    triggered=lambda: self.emit(SIGNAL("open_parent_dir()")))
+                            text=translate('Explorer', "Parent"),
+                            icon=get_icon('up.png'),
+                            triggered=self.treewidget.go_to_parent_directory)
         self.toolbar.addAction(parent_action)
                 
         refresh_action = create_action(self,
@@ -603,8 +662,6 @@ class Test(QDialog):
         hlayout2.addWidget(self.label2)
         self.connect(self.explorer, SIGNAL("open_dir(QString)"),
                      self.label2.setText)
-        self.connect(self.explorer, SIGNAL("open_dir(QString)"),
-                     lambda path: os.chdir(unicode(path)))
         
         hlayout3 = QHBoxLayout()
         vlayout.addLayout(hlayout3)
