@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -17,18 +17,25 @@ from PyQt4.QtCore import QProcess, SIGNAL, QString
 
 # Local imports
 from spyderlib.utils import encoding
+from spyderlib.utils.programs import split_clo
 from spyderlib.config import get_icon
-from spyderlib.widgets.externalshell import (ExternalShellBase,
-                                             add_pathlist_to_PYTHONPATH)
+from spyderlib.widgets.externalshell.baseshell import (ExternalShellBase,
+                                                   add_pathlist_to_PYTHONPATH)
 from spyderlib.widgets.shell import TerminalWidget
 
 
 class ExternalSystemShell(ExternalShellBase):
     """External Shell widget: execute Python script in a separate process"""
     SHELL_CLASS = TerminalWidget
-    def __init__(self, parent=None, wdir=None, path=[]):
+    def __init__(self, parent=None, wdir=None, path=[], light_background=True,
+                 menu_actions=None, show_buttons_inside=True,
+                 show_elapsed_time=True):
         ExternalShellBase.__init__(self, parent, wdir,
-                                   history_filename='.history_ec')
+                                   history_filename='.history',
+                                   light_background=light_background,
+                                   menu_actions=menu_actions,
+                                   show_buttons_inside=show_buttons_inside,
+                                   show_elapsed_time=show_elapsed_time)
         
         # Additional python path list
         self.path = path
@@ -58,7 +65,7 @@ class ExternalSystemShell(ExternalShellBase):
             p_args = ['-i']
             
         if self.arguments:
-            p_args.extend( self.arguments.split(' ') )
+            p_args.extend( split_clo(self.arguments) )
                         
         self.connect(self.process, SIGNAL("readyReadStandardOutput()"),
                      self.write_output)
