@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -39,7 +39,7 @@ from spyderlib.utils.qthelpers import (get_std_icon, create_toolbutton,
                                        translate, get_filetype_icon)
 from spyderlib.config import get_icon
 from spyderlib.widgets.comboboxes import PathComboBox, PatternComboBox
-from spyderlib.widgets import OneColumnTree
+from spyderlib.widgets.onecolumntree import OneColumnTree
 
 
 def abspardir(path):
@@ -178,7 +178,7 @@ class SearchThread(QThread):
             self.stopped = True
 
     def find_files_in_python_path(self):
-        pathlist = os.environ['PYTHONPATH'].split(os.pathsep)
+        pathlist = os.environ.get('PYTHONPATH', '').split(os.pathsep)
         if self.get_pythonpath_callback is not None:
             pathlist += self.get_pythonpath_callback()
         if os.name == "nt":
@@ -194,6 +194,7 @@ class SearchThread(QThread):
                     lcpathlist.append(lcpath)
                     winpathlist.append(path)
             pathlist = winpathlist
+        ok = True
         for path in set(pathlist):
             if osp.isdir(path):
                 ok = self.find_files_in_path(path)
@@ -282,7 +283,7 @@ class SearchThread(QThread):
                             self.results[osp.abspath(fname)] = res
                             for text in self.texts:
                                 found = line.find(text, found+1)
-                                if found>-1:
+                                if found > -1:
                                     break
                             self.nb += 1
             except IOError, (_errno, _strerror):
@@ -325,8 +326,10 @@ class FindOptions(QWidget):
         hlayout1 = QHBoxLayout()
         self.search_text = PatternComboBox(self, search_text,
                                     translate('FindInFiles', "Search pattern"))
-        self.edit_regexp = create_toolbutton(self, get_icon("advanced.png"),
-                         tip=translate('FindInFiles', "Regular expression"))
+        self.edit_regexp = create_toolbutton(self,
+                                             icon=get_icon("advanced.png"),
+                                             tip=translate('FindInFiles',
+                                                        "Regular expression"))
         self.edit_regexp.setCheckable(True)
         self.edit_regexp.setChecked(search_text_regexp)
         self.more_widgets = ()
@@ -337,15 +340,17 @@ class FindOptions(QWidget):
         
         self.ok_button = create_toolbutton(self,
                                 text=translate('FindInFiles', "Search"),
-                                triggered=lambda: self.emit(SIGNAL('find()')),
                                 icon=get_std_icon("DialogApplyButton"),
-                                tip=translate('FindInFiles', "Start search"))
+                                triggered=lambda: self.emit(SIGNAL('find()')),
+                                tip=translate('FindInFiles', "Start search"),
+                                text_beside_icon=True)
         self.connect(self.ok_button, SIGNAL('clicked()'), self.update_combos)
         self.stop_button = create_toolbutton(self,
                                 text=translate('FindInFiles', "Stop"),
-                                triggered=lambda: self.emit(SIGNAL('stop()')),
                                 icon=get_icon("terminate.png"),
-                                tip=translate('FindInFiles', "Stop search"))
+                                triggered=lambda: self.emit(SIGNAL('stop()')),
+                                tip=translate('FindInFiles', "Stop search"),
+                                text_beside_icon=True)
         self.stop_button.setEnabled(False)
         for widget in [self.search_text, self.edit_regexp,
                        self.ok_button, self.stop_button, self.more_options]:
@@ -358,7 +363,8 @@ class FindOptions(QWidget):
         if include_idx is not None and include_idx >= 0 \
            and include_idx < self.include_pattern.count():
             self.include_pattern.setCurrentIndex(include_idx)
-        self.include_regexp = create_toolbutton(self, get_icon("advanced.png"),
+        self.include_regexp = create_toolbutton(self,
+                                            icon=get_icon("advanced.png"),
                                             tip=translate('FindInFiles',
                                                           "Regular expression"))
         self.include_regexp.setCheckable(True)
@@ -370,7 +376,8 @@ class FindOptions(QWidget):
         if exclude_idx is not None and exclude_idx >= 0 \
            and exclude_idx < self.exclude_pattern.count():
             self.exclude_pattern.setCurrentIndex(exclude_idx)
-        self.exclude_regexp = create_toolbutton(self, get_icon("advanced.png"),
+        self.exclude_regexp = create_toolbutton(self,
+                                            icon=get_icon("advanced.png"),
                                             tip=translate('FindInFiles',
                                                           "Regular expression"))
         self.exclude_regexp.setCheckable(True)
@@ -408,7 +415,7 @@ class FindOptions(QWidget):
                      self.dir_combo.setDisabled)
         self.connect(self.hg_manifest, SIGNAL('toggled(bool)'),
                      self.dir_combo.setDisabled)
-        browse = create_toolbutton(self, get_std_icon('DirOpenIcon'),
+        browse = create_toolbutton(self, icon=get_std_icon('DirOpenIcon'),
                                    tip=translate('FindInFiles',
                                                  'Browse a search directory'),
                                    triggered=self.select_directory)
@@ -668,9 +675,16 @@ class ResultsBrowser(OneColumnTree):
             file_item = QTreeWidgetItem(parent_item, [osp.basename(filename)],
                                         QTreeWidgetItem.Type)
             file_item.setIcon(0, get_filetype_icon(filename))
+            colno_dict = {}
+            fname_res = []
             for lineno, colno, line in self.results[filename]:
+                if lineno not in colno_dict:
+                    fname_res.append((lineno, colno, line))
+                colno_dict[lineno] = colno_dict.get(lineno, [])+[str(colno)]
+            for lineno, colno, line in fname_res:
+                colno_str = ",".join(colno_dict[lineno])
                 item = QTreeWidgetItem(file_item,
-                           ["%d (%d): %s" % (lineno, colno, line.rstrip())],
+                           ["%d (%s): %s" % (lineno, colno_str, line.rstrip())],
                            QTreeWidgetItem.Type)
                 item.setIcon(0, get_icon('arrow.png'))
                 self.data[item] = (filename, lineno)
@@ -713,16 +727,16 @@ class FindInFilesWidget(QWidget):
         
         self.result_browser = ResultsBrowser(self)
         
-        collapse_btn = create_toolbutton(self, text_beside_icon=False)
+        collapse_btn = create_toolbutton(self)
         collapse_btn.setDefaultAction(self.result_browser.collapse_all_action)
-        expand_btn = create_toolbutton(self, text_beside_icon=False)
+        expand_btn = create_toolbutton(self)
         expand_btn.setDefaultAction(self.result_browser.expand_all_action)
-        restore_btn = create_toolbutton(self, text_beside_icon=False)
+        restore_btn = create_toolbutton(self)
         restore_btn.setDefaultAction(self.result_browser.restore_action)
-#        collapse_sel_btn = create_toolbutton(self, text_beside_icon=False)
+#        collapse_sel_btn = create_toolbutton(self)
 #        collapse_sel_btn.setDefaultAction(
 #                                self.result_browser.collapse_selection_action)
-#        expand_sel_btn = create_toolbutton(self, text_beside_icon=False)
+#        expand_sel_btn = create_toolbutton(self)
 #        expand_sel_btn.setDefaultAction(
 #                                self.result_browser.expand_selection_action)
         
