@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -12,8 +12,8 @@
 # pylint: disable-msg=R0201
 
 from PyQt4.QtGui import (QHBoxLayout, QGridLayout, QCheckBox, QLabel, QWidget,
-                         QSizePolicy)
-from PyQt4.QtCore import SIGNAL, SLOT, Qt
+                         QSizePolicy, QShortcut, QKeySequence)
+from PyQt4.QtCore import SIGNAL, Qt
 
 import sys
 
@@ -23,6 +23,7 @@ STDOUT = sys.stdout
 # Local imports
 from spyderlib.utils.qthelpers import get_std_icon, create_toolbutton
 from spyderlib.widgets.comboboxes import PatternComboBox
+from spyderlib.config import get_icon
 
 
 class FindReplace(QWidget):
@@ -53,12 +54,16 @@ class FindReplace(QWidget):
         self.connect(self.search_text, SIGNAL("editTextChanged(QString)"),
                      self.text_has_changed)
         
+        self.re_button = create_toolbutton(self, icon=get_icon("advanced.png"),
+                                           tip=self.tr("Regular expression"))
+        self.re_button.setCheckable(True)
+        self.connect(self.re_button, SIGNAL("toggled(bool)"),
+                     lambda state: self.find())
+        
         self.previous_button = create_toolbutton(self,
-                                             text=self.tr("Previous"),
                                              triggered=self.find_previous,
                                              icon=get_std_icon("ArrowBack"))
         self.next_button = create_toolbutton(self,
-                                             text=self.tr("Next"),
                                              triggered=self.find_next,
                                              icon=get_std_icon("ArrowForward"))
         self.connect(self.next_button, SIGNAL('clicked()'),
@@ -67,31 +72,30 @@ class FindReplace(QWidget):
                      self.update_search_combo)
 
         self.case_check = QCheckBox(self.tr("Case Sensitive"))
-        self.connect(self.case_check, SIGNAL("stateChanged(int)"), self.find)
+        self.connect(self.case_check, SIGNAL("stateChanged(int)"),
+                     lambda state: self.find())
         self.words_check = QCheckBox(self.tr("Whole words"))
-        self.connect(self.words_check, SIGNAL("stateChanged(int)"), self.find)
+        self.connect(self.words_check, SIGNAL("stateChanged(int)"),
+                     lambda state: self.find())
 
         hlayout = QHBoxLayout()
-        self.widgets = [self.close_button, self.search_text, self.previous_button,
-                        self.next_button, self.case_check, self.words_check]
+        self.widgets = [self.close_button, self.search_text, self.re_button,
+                        self.previous_button, self.next_button,
+                        self.case_check, self.words_check]
         for widget in self.widgets[1:]:
             hlayout.addWidget(widget)
         glayout.addLayout(hlayout, 0, 1)
 
         # Replace layout
-        replace_with1 = QLabel(self.tr("Replace"))
-        replace_with2 = QLabel()
-        font = replace_with2.font()
-        font.setBold(True)
-        replace_with2.setFont(font)
-        replace_with3 = QLabel(self.tr("with:"))
+        replace_with = QLabel(self.tr("Replace with:"))
         self.replace_text = PatternComboBox(self, adjust_to_minimum=False,
                                             tip=self.tr("Replace string"))
         
         self.replace_button = create_toolbutton(self,
                                      text=self.tr("Replace/find"),
+                                     icon=get_std_icon("DialogApplyButton"),
                                      triggered=self.replace_find,
-                                     icon=get_std_icon("DialogApplyButton"))
+                                     text_beside_icon=True)
         self.connect(self.replace_button, SIGNAL('clicked()'),
                      self.update_replace_combo)
         self.connect(self.replace_button, SIGNAL('clicked()'),
@@ -100,12 +104,10 @@ class FindReplace(QWidget):
         self.all_check = QCheckBox(self.tr("Replace all"))
         
         self.replace_layout = QHBoxLayout()
-        widgets = [replace_with1, replace_with2, replace_with3,
-                   self.replace_text, self.replace_button, self.all_check]
+        widgets = [replace_with, self.replace_text, self.replace_button,
+                   self.all_check]
         for widget in widgets:
             self.replace_layout.addWidget(widget)
-        self.connect(self.search_text, SIGNAL("editTextChanged(QString)"),
-                     replace_with2, SLOT("setText(QString)"))
         glayout.addLayout(self.replace_layout, 1, 1)
         self.widgets.extend(widgets)
         self.replace_widgets = widgets
@@ -115,32 +117,41 @@ class FindReplace(QWidget):
         
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         
+        self.findnext_sc = QShortcut(QKeySequence("F3"), parent, self.find_next)
+        self.findnext_sc.setContext(Qt.WidgetWithChildrenShortcut)
+        self.findprev_sc = QShortcut(QKeySequence("Shift+F3"), parent,
+                                     self.find_previous)
+        self.findprev_sc.setContext(Qt.WidgetWithChildrenShortcut)
+        self.togglefind_sc = QShortcut(QKeySequence("Ctrl+F"), parent,
+                                       self.show)
+        self.togglefind_sc.setContext(Qt.WidgetWithChildrenShortcut)
+        self.togglereplace_sc = QShortcut(QKeySequence("Ctrl+H"), parent,
+                                          self.toggle_replace_widgets)
+        self.togglereplace_sc.setContext(Qt.WidgetWithChildrenShortcut)
+        
+        escape_sc = QShortcut(QKeySequence("Escape"), parent, self.hide)
+        escape_sc.setContext(Qt.WidgetWithChildrenShortcut)
+        
+    def get_shortcut_data(self):
+        """
+        Returns shortcut data, a list of tuples (shortcut, text, default)
+        shortcut (QShortcut or QAction instance)
+        text (string): action/shortcut description
+        default (string): default key sequence
+        """
+        return [(self.findnext_sc, "Find next", "F3"),
+                (self.findprev_sc, "Find previous", "Shift+F3"),
+                (self.togglefind_sc, "Find text", "Ctrl+F"),
+                (self.togglereplace_sc, "Replace text", "Ctrl+H"),]
+        
     def update_search_combo(self):
         self.search_text.lineEdit().emit(SIGNAL('returnPressed()'))
         
     def update_replace_combo(self):
         self.replace_text.lineEdit().emit(SIGNAL('returnPressed()'))
-        
-    def keyPressEvent(self, event):
-        """Reimplemented to handle key events"""
-        ctrl = event.modifiers() & Qt.ControlModifier
-        if event.key() == Qt.Key_Escape:
-            self.hide()
-            event.accept()
-            return
-        elif event.key() == Qt.Key_F3:
-            # Find next
-            self.find_next()
-            self.editor.setFocus()
-            event.accept()
-        elif event.key() == Qt.Key_F and ctrl:
-            # Toggle find widgets
-            if self.isVisible():
-                self.hide()
-            else:
-                self.show()
-            event.accept()
-        elif event.key() == Qt.Key_H and ctrl and self.enable_replace:
+    
+    def toggle_replace_widgets(self):
+        if self.enable_replace:
             # Toggle replace widgets
             if self.replace_widgets[0].isVisible():
                 self.hide_replace()
@@ -148,16 +159,13 @@ class FindReplace(QWidget):
             else:
                 self.show_replace()
                 self.replace_text.setFocus()
-            event.accept()
-        else:
-            event.ignore()
         
     def show(self):
         """Overrides Qt Method"""
         QWidget.show(self)
         self.emit(SIGNAL("visibility_changed(bool)"), True)
         if self.editor is not None:
-            text = self.editor.selectedText()
+            text = self.editor.get_selected_text()
             if len(text) > 0:
                 self.search_text.setEditText(text)
                 self.search_text.lineEdit().selectAll()
@@ -177,6 +185,7 @@ class FindReplace(QWidget):
         
     def show_replace(self):
         """Show replace widgets"""
+        self.show()
         for widget in self.replace_widgets:
             widget.show()
             
@@ -198,23 +207,27 @@ class FindReplace(QWidget):
     def set_editor(self, editor, refresh=True):
         """
         Set associated editor/web page:
-            qtebase.TextEditBaseWidget
-            qscibase.TextEditBaseWidget
+            codeeditor.base.TextEditBaseWidget
             browser.WebView
         """
         self.editor = editor
         from PyQt4.QtWebKit import QWebView
         self.words_check.setVisible(not isinstance(editor, QWebView))
+        self.re_button.setVisible(not isinstance(editor, QWebView))
         if refresh:
             self.refresh()
         
     def find_next(self):
         """Find next occurence"""
-        return self.find(changed=False, forward=True)
+        state = self.find(changed=False, forward=True)
+        self.editor.setFocus()
+        return state
         
     def find_previous(self):
         """Find previous occurence"""
-        return self.find(changed=False, forward=False)
+        state = self.find(changed=False, forward=False)
+        self.editor.setFocus()
+        return state
         
     def text_has_changed(self, text):
         """Find text has changed"""
@@ -229,7 +242,8 @@ class FindReplace(QWidget):
         else:
             found = self.editor.find_text(text, changed, forward,
                                           case=self.case_check.isChecked(),
-                                          words=self.words_check.isChecked())
+                                          words=self.words_check.isChecked(),
+                                          regexp=self.re_button.isChecked())
             self.search_text.lineEdit().setStyleSheet(self.STYLE[found])
             return found
             
@@ -242,8 +256,8 @@ class FindReplace(QWidget):
             while True:
                 if first:
                     # First found
-                    if self.editor.hasSelectedText() \
-                       and self.editor.selectedText() == search_text:
+                    if self.editor.has_selected_text() and \
+                       self.editor.get_selected_text() == unicode(search_text):
                         # Text was already found, do nothing
                         pass
                     else:
@@ -274,4 +288,4 @@ class FindReplace(QWidget):
                 if not self.all_check.isChecked():
                     break
             self.all_check.setCheckState(Qt.Unchecked)
-            
\ No newline at end of file
+            
