@@ -10,7 +10,7 @@ from PyQt4.QtGui import (QDialog, QVBoxLayout, QLabel, QHBoxLayout, QMenu,
                          QWidget, QTreeWidgetItem, QFileIconProvider,
                          QMessageBox, QInputDialog, QLineEdit, QFileDialog)
 from PyQt4.QtCore import (Qt, SIGNAL, QFileInfo, QFileSystemWatcher,
-                          QUrl)
+                          QUrl, QTimer)
 
 import os, sys, re, shutil, cPickle
 import os.path as osp
@@ -20,17 +20,22 @@ STDOUT = sys.stdout
 STDERR = sys.stderr
 
 # Local imports
-from spyderlib.utils import count_lines, rename_file, remove_file, move_file
+from spyderlib.utils import (count_lines, rename_file, remove_file, move_file,
+                             programs, log_last_error)
 from spyderlib.utils.qthelpers import (get_std_icon, translate, create_action,
                                        create_toolbutton, add_actions,
                                        set_item_user_text)
 from spyderlib.utils.qthelpers import get_item_user_text as get_item_path
-from spyderlib.config import get_icon, get_image_path
-from spyderlib.widgets import OneColumnTree
+from spyderlib.config import get_icon, get_image_path, get_conf_path
+from spyderlib.widgets.onecolumntree import OneColumnTree
 from spyderlib.widgets.formlayout import fedit
 from spyderlib.widgets.pathmanager import PathManager
 
 
+DEBUG = False
+LOG_FILENAME = get_conf_path('rope.log')
+
+
 def listdir(path, include='.', exclude=r'\.pyc$|^\.', show_all=False,
             folders_only=False):
     """List files and directories"""
@@ -89,7 +94,20 @@ def get_dir_icon(dirname, expanded=False, pythonpath=False, root=False):
             return get_icon(prefix+'folder_expanded.png')
         else:
             return get_icon(prefix+'folder_collapsed.png')
-            
+
+
+ROPE_PREFS = {'ignore_syntax_errors': True,
+              'ignore_bad_imports': True,
+              'soa_followed_calls': 2,
+              'extension_modules': [
+        "PyQt4", "PyQt4.QtGui", "QtGui", "PyQt4.QtCore", "QtCore",
+        "PyQt4.QtScript", "QtScript", "os.path", "numpy", "scipy", "PIL",
+        "OpenGL", "array", "audioop", "binascii", "cPickle", "cStringIO",
+        "cmath", "collections", "datetime", "errno", "exceptions", "gc",
+        "imageop", "imp", "itertools", "marshal", "math", "mmap", "msvcrt",
+        "nt", "operator", "os", "parser", "rgbimg", "signal", "strop", "sys",
+        "thread", "time", "wx", "wxPython", "xxsubtype", "zipimport", "zlib",
+                    ]}            
 
 class Project(object):
     """Spyder project"""
@@ -105,6 +123,7 @@ class Project(object):
         self.pythonpath = []
         self.folders = set()
         self.opened = True
+        self.rope_project = None
 
         config_path = self.__get_project_config_path()
         if not osp.exists(config_path):
@@ -126,7 +145,7 @@ class Project(object):
         return osp.join(self.root_path, self.CONFIG_NAME)
         
     def load(self):
-        data = cPickle.load(file(self.__get_project_config_path()))
+        data = cPickle.loads(file(self.__get_project_config_path(), 'U').read())
         # Compatibilty with old project explorer file format:
         if 'relative_pythonpath' not in data:
             print >>STDERR, "Warning: converting old configuration file " \
@@ -136,13 +155,13 @@ class Project(object):
         for attr in self.CONFIG_ATTR:
             setattr(self, attr, data[attr])
         self.save()
+        self.create_rope_project()
     
     def save(self):
         data = {}
         for attr in self.CONFIG_ATTR:
             data[attr] = getattr(self, attr)
-        cPickle.dump(data, file(self.__get_project_config_path(), 'w'),
-                     cPickle.HIGHEST_PROTOCOL)
+        cPickle.dump(data, file(self.__get_project_config_path(), 'w'))
         
     def delete(self):
         os.remove(self.__get_project_config_path())
@@ -162,6 +181,105 @@ class Project(object):
     def get_root_item(self):
         return self.items[self.root_path]
         
+    #------rope integration
+    def create_rope_project(self):
+        try:
+            import rope.base.project
+            self.rope_project = rope.base.project.Project(self.root_path,
+                                                          **ROPE_PREFS)
+        except ImportError:
+            self.rope_project = None
+            if DEBUG:
+                log_last_error(LOG_FILENAME,
+                               "create_rope_project: %r" % self.root_path)
+        except TypeError:
+            # Compatibility with new Mercurial API (>= 1.3).
+            # New versions of rope (> 0.9.2) already handle this issue
+            self.rope_project = None
+            if DEBUG:
+                log_last_error(LOG_FILENAME,
+                               "create_rope_project: %r" % self.root_path)
+        self.validate_rope_project()
+        
+    def close_rope_project(self):
+        if self.rope_project is not None:
+            self.rope_project.close()
+            
+    def validate_rope_project(self):
+        if self.rope_project is not None:
+            self.rope_project.validate(self.rope_project.root)
+        
+    def get_completion_list(self, source_code, offset, filename):
+        if self.rope_project is None:
+            return []
+        import rope.base.libutils
+        try:
+            resource = rope.base.libutils.path_to_resource(self.rope_project,
+                                                           filename)
+        except Exception, _error:
+            if DEBUG:
+                log_last_error(LOG_FILENAME, "path_to_resource: %r" % filename)
+            resource = None
+        try:
+            import rope.contrib.codeassist
+            proposals = rope.contrib.codeassist.code_assist(self.rope_project,
+                                                source_code, offset, resource)
+            proposals = rope.contrib.codeassist.sorted_proposals(proposals)
+            return [proposal.name for proposal in proposals]
+        except Exception, _error:
+            if DEBUG:
+                log_last_error(LOG_FILENAME, "get_completion_list")
+            return []
+
+    def get_calltip_text(self, source_code, offset, filename):
+        if self.rope_project is None:
+            return []
+        import rope.base.libutils
+        try:
+            resource = rope.base.libutils.path_to_resource(self.rope_project,
+                                                           filename)
+        except Exception, _error:
+            if DEBUG:
+                log_last_error(LOG_FILENAME, "path_to_resource: %r" % filename)
+            resource = None
+        try:
+            import rope.contrib.codeassist
+            cts = rope.contrib.codeassist.get_calltip(self.rope_project,
+                                        source_code, offset, resource,
+                                        ignore_unknown=True, remove_self=True)
+            if cts is not None:
+                return [cts]
+            else:
+                return []
+        except Exception, _error:
+            if DEBUG:
+                log_last_error(LOG_FILENAME, "get_calltip_text")
+            return []
+    
+    def get_definition_location(self, source_code, offset, filename):
+        if self.rope_project is None:
+            return (None, None)
+        import rope.base.libutils
+        try:
+            resource = rope.base.libutils.path_to_resource(self.rope_project,
+                                                           filename)
+        except Exception, _error:
+            if DEBUG:
+                log_last_error(LOG_FILENAME, "path_to_resource: %r" % filename)
+            resource = None
+        try:
+            import rope.contrib.codeassist
+            resource, lineno = rope.contrib.codeassist.get_definition_location(
+                            self.rope_project, source_code, offset, resource)
+            if resource is not None:
+                filename = resource.real_path
+            return filename, lineno
+        except Exception, _error:
+            if DEBUG:
+                log_last_error(LOG_FILENAME, "get_definition_location")
+            return (None, None)
+        
+    #------Misc.
     def get_related_projects(self):
         """Return related projects path list"""
         return self.related_projects
@@ -178,13 +296,28 @@ class Project(object):
         else:
             return get_icon(icon_path)
         
-    def set_opened(self, opened):
-        self.opened = opened
+    def open(self):
+        """Open project"""
+        self.opened = True
         self.save()
+        self.create_rope_project()
+        
+    def close(self):
+        """Close project"""
+        self.opened = False
+        self.save()
+        self.close_rope_project()
         
     def is_opened(self):
         return self.opened
+    
+    def is_file_in_project(self, fname):
+        """Return True if file *fname* is in one of the project subfolders"""
+        norm = osp.normcase if os.name == 'nt' else osp.normpath
+        fixpath = lambda path: norm(osp.abspath(path))
+        return fixpath(fname).startswith(fixpath(self.root_path))
         
+    #------Python Path
     def get_pythonpath(self):
         return self.pythonpath[:] # return a copy of pythonpath attribute
         
@@ -204,6 +337,7 @@ class Project(object):
         self.__update_pythonpath_icons()
         self.save()
         
+    #------Tree widget items
     def get_items(self):
         return self.items.values()
         
@@ -421,6 +555,8 @@ class ExplorerTreeWidget(OneColumnTree):
         OneColumnTree.__init__(self, parent)
         self.parent_widget = parent
         
+        self.default_project = None
+        
         self.projects = []
         self.__update_title()
         
@@ -450,6 +586,23 @@ class ExplorerTreeWidget(OneColumnTree):
         self.setAcceptDrops(True)
         self.setAutoExpandDelay(500)
         
+    def closing_widget(self):
+        """Perform actions before widget is closed"""
+        for project in self.projects+[self.default_project]:
+            if project.is_opened():
+                project.close_rope_project()
+
+    def get_source_project(self, fname):
+        """Return project which contains source *fname*"""
+        return self.default_project # See Issue 490
+        # See Issue 490: always using the default project to avoid performance 
+        # issues with 'rope' when dealing with projects containing large files
+#        for project in self.projects:
+#            if project.is_file_in_project(fname):
+#                return project
+#        else:
+#            return self.default_project
+        
     def get_project_from_name(self, name):
         for project in self.projects:
             if project.get_name() == name:
@@ -463,11 +616,14 @@ class ExplorerTreeWidget(OneColumnTree):
         return pythonpath
         
     def setup(self, include='.', exclude=r'\.pyc$|^\.', show_all=False,
-              valid_types=['.py', '.pyw']):
+              valid_types=['.py', '.pyw'], default_project_path=None):
         self.include = include
         self.exclude = exclude
         self.show_all = show_all
         self.valid_types = valid_types
+        assert default_project_path is not None
+        self.default_project = Project(default_project_path)
+        self.default_project.load()
         
     def __get_project_from_item(self, item):
         for project in self.projects:
@@ -619,6 +775,12 @@ class ExplorerTreeWidget(OneColumnTree):
             if any_folder_in_path:
                 actions += [remove_from_path_act]
             actions += [None, winexp_act, terminal_act, interpreter_act]
+            if programs.is_module_installed("IPython"):
+                ipython_act = create_action(self,
+                        text=translate('ProjectExplorer', "Open IPython here"),
+                        icon="ipython.png",
+                        triggered=lambda: self.open_ipython(items))
+                actions.append(ipython_act)
         actions += [None, properties_act]
         
         return actions
@@ -665,7 +827,7 @@ class ExplorerTreeWidget(OneColumnTree):
             self.rename(self.currentItem())
             event.accept()
         else:
-            super(ExplorerTreeWidget, self).keyPressEvent(event)
+            OneColumnTree.keyPressEvent(self, event)
 
     def activated(self):
         """Reimplement OneColumnTree method"""
@@ -755,7 +917,7 @@ class ExplorerTreeWidget(OneColumnTree):
         if not isinstance(projects, (tuple, list)):
             projects = [projects]
         for project in projects:
-            project.set_opened(True)
+            project.open()
             project.refresh(self, self.include, self.exclude, self.show_all)
             related_projects = project.get_related_projects()
             if open_related:
@@ -769,7 +931,7 @@ class ExplorerTreeWidget(OneColumnTree):
         if not isinstance(projects, (tuple, list)):
             projects = [projects]
         for project in projects:
-            project.set_opened(False)
+            project.close()
             project.refresh(self, self.include, self.exclude, self.show_all)
         self.parent_widget.emit(SIGNAL("pythonpath_changed()"))
         
@@ -987,18 +1149,21 @@ class ExplorerTreeWidget(OneColumnTree):
         title = translate('ProjectExplorer', 'New folder')
         subtitle = translate('ProjectExplorer', 'Folder name:')
         self.__create_new_folder(item, title, subtitle, is_package=False)
+        QTimer.singleShot(500, self.refresh)
     
     def new_module(self, item):
         title = translate('ProjectExplorer', "New module")
-        filters = translate('Explorer', "Python scripts")+" (*.py *.pyw)"
+        filters = translate('Explorer', "Python scripts")+" (*.py *.pyw *.ipy)"
         create_func = lambda fname: self.parent_widget.emit( \
                                      SIGNAL("create_module(QString)"), fname)
         self.__create_new_file(item, title, filters, create_func)
+        QTimer.singleShot(500, self.refresh)
     
     def new_package(self, item):
         title = translate('ProjectExplorer', 'New package')
         subtitle = translate('ProjectExplorer', 'Package name:')
         self.__create_new_folder(item, title, subtitle, is_package=True)
+        QTimer.singleShot(500, self.refresh)
     
     def open_file_from_menu(self, items):
         if not isinstance(items, (tuple, list)):
@@ -1109,6 +1274,7 @@ class ExplorerTreeWidget(OneColumnTree):
                                   "<b>Unable to %1 <i>%2</i></b>"
                                   "<br><br>Error message:<br>%3") \
                         .arg(action_str).arg(fname).arg(str(error)))
+        QTimer.singleShot(500, self.refresh)
     
     def add_path_to_project_pythonpath(self, project, path):
         pathlist = project.get_pythonpath()
@@ -1203,6 +1369,10 @@ class ExplorerTreeWidget(OneColumnTree):
     def open_interpreter(self, items):
         for path in sorted([get_item_path(_it) for _it in items]):
             self.parent_widget.emit(SIGNAL("open_interpreter(QString)"), path)
+            
+    def open_ipython(self, items):
+        for path in sorted([get_item_path(_it) for _it in items]):
+            self.parent_widget.emit(SIGNAL("open_ipython(QString)"), path)
         
     def refresh(self, clear=True):
 #        if clear:
@@ -1217,7 +1387,7 @@ class ExplorerTreeWidget(OneColumnTree):
     
     def mimeData(self, items):
         """Reimplement Qt method"""
-        data = super(ExplorerTreeWidget, self).mimeData(items)
+        data = OneColumnTree.mimeData(self, items)
         data.setUrls([QUrl(get_item_path(item)) for item in items])
         return data
     
@@ -1322,27 +1492,29 @@ class ProjectExplorerWidget(QWidget):
         SIGNAL("removed(QString)")
     """
     def __init__(self, parent=None, include='.', exclude=r'\.pyc$|^\.',
-                 show_all=False, valid_types=['.py', '.pyw']):
+                 show_all=False, valid_types=['.py', '.pyw'],
+                 default_project_path=None):
         QWidget.__init__(self, parent)
         
         self.treewidget = ExplorerTreeWidget(self)
         self.treewidget.setup(include=include, exclude=exclude,
-                              show_all=show_all, valid_types=valid_types)
+                              show_all=show_all, valid_types=valid_types,
+                              default_project_path=default_project_path)
 
-        filters_action = create_toolbutton(self, get_icon('filter.png'),
+        filters_action = create_toolbutton(self, icon=get_icon('filter.png'),
                                            tip=translate('ProjectExplorer',
                                                    "Edit filename filter..."),
                                            triggered=self.edit_filter)
         # Show all files
-        all_action = create_toolbutton(self, get_icon('show_all.png'),
+        all_action = create_toolbutton(self, icon=get_icon('show_all.png'),
                                        tip=translate('ProjectExplorer',
                                                      "Show all files"),
                                        toggled=self.toggle_all)
         all_action.setChecked(show_all)
         
         refresh_btn = create_toolbutton(self,
-                        tip=translate('ProjectExplorer', "Refresh"),
                         icon=get_icon('reload.png'),
+                        tip=translate('ProjectExplorer', "Refresh"),
                         triggered=lambda: self.treewidget.refresh(clear=True))
         
         collapse_btn = create_toolbutton(self, text_beside_icon=False)
@@ -1363,6 +1535,10 @@ class ProjectExplorerWidget(QWidget):
         layout.addLayout(btn_layout)
         self.setLayout(layout)
         
+    def closing_widget(self):
+        """Perform actions before widget is closed"""
+        self.treewidget.closing_widget()
+        
     def add_project(self, project):
         return self.treewidget.add_project(project)
         
@@ -1379,6 +1555,10 @@ class ProjectExplorerWidget(QWidget):
 
     def get_pythonpath(self):
         return self.treewidget.get_pythonpath()
+    
+    def get_source_project(self, fname):
+        """Return project which contains source *fname*"""
+        return self.treewidget.get_source_project(fname)
 
     def edit_filter(self):
         """Edit include/exclude filter"""
