@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -11,12 +11,11 @@
 # pylint: disable-msg=R0911
 # pylint: disable-msg=R0201
 
-import sys, os, time, string
-import os.path as osp
+import sys, os, time, os.path as osp, re
 
 from PyQt4.QtGui import (QMenu, QApplication, QCursor, QToolTip, QKeySequence,
                          QFileDialog, QMessageBox, QMouseEvent, QTextCursor,
-                         QTextCharFormat)
+                         QTextCharFormat, QShortcut)
 from PyQt4.QtCore import Qt, QString, QCoreApplication, SIGNAL, pyqtProperty
 
 # For debugging purpose:
@@ -25,16 +24,12 @@ STDERR = sys.stderr
 
 # Local import
 from spyderlib import __version__
-from spyderlib.utils import encoding
 from spyderlib.config import CONF, get_icon, get_conf_path
+from spyderlib.utils import encoding, get_error_match
 from spyderlib.utils.dochelpers import getobj
 from spyderlib.utils.qthelpers import (keybinding, create_action, add_actions,
                                        restore_keyevent, translate)
-from spyderlib.widgets.shellhelpers import get_error_match
-# Shell widgets are based on QPlainTextEdit (defined in 'qtebase' module) 
-# instead of QsciScintilla (defined in 'qscibase' module - used in the editor 
-# widgets) because QPlainTextEdit is much faster:
-from spyderlib.widgets.qteditor.qtebase import ConsoleBaseWidget
+from spyderlib.widgets.codeeditor.base import ConsoleBaseWidget
 
 
 HISTORY_FILENAMES = []
@@ -85,11 +80,8 @@ class ShellBaseWidget(ConsoleBaseWidget):
         # Give focus to widget
         self.setFocus()
                 
-    def setup(self):
-        """Reimplement ConsoleBaseWidget method"""
-        ConsoleBaseWidget.setup(self)
-        self.set_caret(color=Qt.darkGray, width=2)
-        self.remove_margins() # Suppressing Scintilla margins
+        # Cursor width
+        self.setCursorWidth( CONF.get('shell_appearance', 'cursor/width') )
         
     def toggle_wrap_mode(self, enable):
         """Reimplement ConsoleBaseWidget method: 'word' -> 'character'"""
@@ -149,7 +141,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
           
     def contextMenuEvent(self, event):
         """Reimplement Qt method"""
-        state = self.hasSelectedText()
+        state = self.has_selected_text()
         self.copy_action.setEnabled(state)
         self.cut_action.setEnabled(state)
         self.delete_action.setEnabled(state)
@@ -230,23 +222,22 @@ class ShellBaseWidget(ConsoleBaseWidget):
     #------ Copy / Keyboard interrupt
     def copy(self):
         """Copy text to clipboard... or keyboard interrupt"""
-        if self.hasSelectedText():
-            text = unicode(self.selectedText()).replace(u"\u2029", os.linesep)
-            QApplication.clipboard().setText(text)
+        if self.has_selected_text():
+            ConsoleBaseWidget.copy(self)
         else:
             self.emit(SIGNAL("keyboard_interrupt()"))
             
     def cut(self):
         """Cut text"""
         self.check_selection()
-        if self.hasSelectedText():
+        if self.has_selected_text():
             ConsoleBaseWidget.cut(self)
 
     def delete(self):
         """Remove selected text"""
         self.check_selection()
-        if self.hasSelectedText():
-            ConsoleBaseWidget.removeSelectedText(self)
+        if self.has_selected_text():
+            ConsoleBaseWidget.remove_selected_text(self)
         
     def save_historylog(self):
         """Save current history log (all text in console)"""
@@ -258,7 +249,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
         if filename:
             filename = osp.normpath(unicode(filename))
             try:
-                encoding.write(unicode(self.text()), filename)
+                encoding.write(unicode(self.get_text_with_eol()), filename)
                 self.historylog_filename = filename
                 CONF.set('main', 'historylog_filename', filename)
             except EnvironmentError, error:
@@ -295,7 +286,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
         """
         Reimplement Qt Method
         Basic keypress event handler
-        (reimplemented in InteractiveShell to add more sophisticated features)
+        (reimplemented in InternalShell to add more sophisticated features)
         """
         if self.preprocess_keyevent(event):
             # Event was accepted in self.preprocess_keyevent
@@ -322,13 +313,13 @@ class ShellBaseWidget(ConsoleBaseWidget):
         
     def postprocess_keyevent(self, event):
         """Post-process keypress event:
-        in InteractiveShell, this is method is called when shell is ready"""
+        in InternalShell, this is method is called when shell is ready"""
         event, text, key, ctrl, shift = restore_keyevent(event)
         
         # Is cursor on the last line? and after prompt?
         if len(text):
             #XXX: Shouldn't it be: `if len(unicode(text).strip(os.linesep))` ?
-            if self.hasSelectedText():
+            if self.has_selected_text():
                 self.check_selection()
             self.restrict_cursor_position(self.current_prompt_pos, 'eof')
             
@@ -339,14 +330,13 @@ class ShellBaseWidget(ConsoleBaseWidget):
                 self._key_enter()
             # add and run selection
             else:
-                text = self.selectedText()
-                self.insert_text(text, at_end=True)
+                self.insert_text(self.get_selected_text(), at_end=True)
             event.accept()
             
         elif key == Qt.Key_Delete:
-            if self.hasSelectedText():
+            if self.has_selected_text():
                 self.check_selection()
-                self.removeSelectedText()
+                self.remove_selected_text()
             elif self.is_cursor_on_last_line():
                 self.stdkey_clear()
             event.accept()
@@ -358,6 +348,10 @@ class ShellBaseWidget(ConsoleBaseWidget):
         elif key == Qt.Key_Tab:
             self._key_tab()
             event.accept()
+            
+        elif key == Qt.Key_Space and ctrl:
+            self._key_ctrl_space()
+            event.accept()
 
         elif key == Qt.Key_Left:
             event.accept()
@@ -389,7 +383,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
                 self.set_cursor_position('eof')
             y_cursor = self.get_coordinates(cursor_position)[1]
             y_prompt = self.get_coordinates(self.current_prompt_pos)[1]
-            if self.is_completion_widget_visible() or y_cursor > y_prompt:
+            if y_cursor > y_prompt:
                 self.stdkey_up(shift)
             else:
                 self.browse_history(backward=True)
@@ -400,7 +394,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
                 self.set_cursor_position('eof')
             y_cursor = self.get_coordinates(cursor_position)[1]
             y_end = self.get_coordinates('eol')[1]
-            if self.is_completion_widget_visible() or y_cursor < y_end:
+            if y_cursor < y_end:
                 self.stdkey_down(shift)
             else:
                 self.browse_history(backward=False)
@@ -436,27 +430,18 @@ class ShellBaseWidget(ConsoleBaseWidget):
             self.selectAll()
             event.accept()
                 
-        elif key == Qt.Key_Question and not self.hasSelectedText():
+        elif key == Qt.Key_Question and not self.has_selected_text():
             self._key_question(text)
             event.accept()
             
-        elif key == Qt.Key_ParenLeft and not self.hasSelectedText():
+        elif key == Qt.Key_ParenLeft and not self.has_selected_text():
             self._key_parenleft(text)
             event.accept()
             
-        elif key == Qt.Key_Period and not self.hasSelectedText():
+        elif key == Qt.Key_Period and not self.has_selected_text():
             self._key_period(text)
             event.accept()
 
-        elif ((key == Qt.Key_Plus) and ctrl) \
-             or ((key==Qt.Key_Equal) and shift and ctrl):
-            self.zoomIn()
-            event.accept()
-
-        elif (key == Qt.Key_Minus) and ctrl:
-            self.zoomOut()
-            event.accept()
-
         elif text.length() and not self.isReadOnly():
             self.hist_wholeline = False
             self.insert_text(text)
@@ -480,6 +465,8 @@ class ShellBaseWidget(ConsoleBaseWidget):
         raise NotImplementedError
     def _key_tab(self):
         raise NotImplementedError
+    def _key_ctrl_space(self):
+        raise NotImplementedError
     def _key_home(self, shift):
         raise NotImplementedError
     def _key_end(self, shift):
@@ -649,7 +636,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
         event: the mouse press event (QMouseEvent)
         """
         if event.button() == Qt.MidButton:
-            text = self.selectedText()
+            text = self.get_selected_text()
             # Simulating left mouse button:
             event = QMouseEvent(QMouseEvent.MouseButtonPress, event.pos(),
                                 Qt.LeftButton, Qt.LeftButton, Qt.NoModifier)
@@ -705,6 +692,7 @@ class PythonShellWidget(ShellBaseWidget):
         ShellBaseWidget.__init__(self, parent, history_filename, debug, profile)
         
         self.inspector = None
+        self.inspector_enabled = True
         
         # Allow raw_input support:
         self.input_loop = None
@@ -713,6 +701,22 @@ class PythonShellWidget(ShellBaseWidget):
         # Mouse cursor
         self.__cursor_changed = False
 
+        # Local shortcuts
+        self.inspectsc = QShortcut(QKeySequence("Ctrl+I"), self,
+                                   self.inspect_current_object)
+        self.inspectsc.setContext(Qt.WidgetWithChildrenShortcut)
+        
+    def get_shortcut_data(self):
+        """
+        Returns shortcut data, a list of tuples (shortcut, text, default)
+        shortcut (QShortcut or QAction instance)
+        text (string): action/shortcut description
+        default (string): default key sequence
+        """
+        return [
+                (self.inspectsc, "Inspect current object", "Ctrl+I"),
+                ]
+
 
     #------ Context menu
     def setup_context_menu(self):
@@ -743,13 +747,13 @@ class PythonShellWidget(ShellBaseWidget):
           
     def contextMenuEvent(self, event):
         """Reimplements ShellBaseWidget method"""
-        state = self.hasSelectedText()
+        state = self.has_selected_text()
         self.copy_without_prompts_action.setEnabled(state)
         ShellBaseWidget.contextMenuEvent(self, event)
         
     def copy_without_prompts(self):
         """Copy text to clipboard without prompts"""
-        text = unicode(self.selectedText()).replace(u"\u2029", os.linesep)
+        text = self.get_selected_text()
         lines = text.split(os.linesep)
         for index, line in enumerate(lines):
             if line.startswith('>>> ') or line.startswith('... '):
@@ -763,7 +767,7 @@ class PythonShellWidget(ShellBaseWidget):
         """Go to error"""
         ConsoleBaseWidget.mouseReleaseEvent(self, event)            
         text = self.get_line_at(event.pos())
-        if get_error_match(text) and not self.hasSelectedText():
+        if get_error_match(text) and not self.has_selected_text():
             self.emit(SIGNAL("go_to_error(QString)"), text)
 
     def mouseMoveEvent(self, event):
@@ -795,12 +799,6 @@ class PythonShellWidget(ShellBaseWidget):
         if QToolTip.isVisible():
             _event, _text, key, _ctrl, _shift = restore_keyevent(event)
             self.hide_tooltip_if_necessary(key)
-            
-    def _key_enter(self):
-        if self.is_completion_widget_visible() and self.codecompletion_enter:
-            self.stdkey_tab()
-        else:
-            ShellBaseWidget._key_enter(self)
                 
     def _key_other(self, text):
         """1 character key"""
@@ -809,9 +807,9 @@ class PythonShellWidget(ShellBaseWidget):
                 
     def _key_backspace(self, cursor_position):
         """Action for Backspace key"""
-        if self.hasSelectedText():
+        if self.has_selected_text():
             self.check_selection()
-            self.removeSelectedText()
+            self.remove_selected_text()
         elif self.current_prompt_pos == cursor_position:
             # Avoid deleting prompt
             return
@@ -824,38 +822,35 @@ class PythonShellWidget(ShellBaseWidget):
                 
     def _key_tab(self):
         """Action for TAB key"""
-        if self.is_completion_widget_visible():
-            self.stdkey_tab()
-        elif self.is_cursor_on_last_line():
+        if self.is_cursor_on_last_line():
             empty_line = not self.get_current_line_to_cursor().strip()
             if empty_line:
                 self.stdkey_tab()
             else:
-                self.show_code_completion()
+                self.show_code_completion(automatic=False)
+                
+    def _key_ctrl_space(self):
+        """Action for Ctrl+Space"""
+        if not self.is_completion_widget_visible():
+            self.show_code_completion(automatic=False)
                 
     def _key_home(self, shift):
         """Action for Home key"""
-        if self.is_completion_widget_visible():
-            self.completion_widget_home()
-        elif self.is_cursor_on_last_line():
+        if self.is_cursor_on_last_line():
             self.stdkey_home(shift, self.current_prompt_pos)
                 
     def _key_end(self, shift):
         """Action for End key"""
-        if self.is_completion_widget_visible():
-            self.completion_widget_end()
-        elif self.is_cursor_on_last_line():
+        if self.is_cursor_on_last_line():
             self.stdkey_end(shift)
                 
     def _key_pageup(self):
         """Action for PageUp key"""
-        if self.is_completion_widget_visible():
-            self.completion_widget_pageup()
+        pass
     
     def _key_pagedown(self):
         """Action for PageDown key"""
-        if self.is_completion_widget_visible():
-            self.completion_widget_pagedown()
+        pass
                 
     def _key_escape(self):
         """Action for ESCAPE key"""
@@ -887,7 +882,7 @@ class PythonShellWidget(ShellBaseWidget):
             # Enable auto-completion only if last token isn't a float
             last_obj = self.get_last_obj()
             if last_obj and not last_obj.isdigit():
-                self.show_code_completion()
+                self.show_code_completion(automatic=True)
 
 
     #------ Paste
@@ -896,7 +891,7 @@ class PythonShellWidget(ShellBaseWidget):
         text = unicode(QApplication.clipboard().text())
         if len(text.splitlines()) > 1:
             # Multiline paste
-            self.removeSelectedText() # Remove selection, eventually
+            self.remove_selected_text() # Remove selection, eventually
             end = self.get_current_line_from_cursor()
             lines = self.get_current_line_to_cursor() + text + end
             self.clear_line()
@@ -908,15 +903,8 @@ class PythonShellWidget(ShellBaseWidget):
     
   
     #------ Code Completion / Calltips        
-    def show_completion_list(self, completions, completion_text=""):
-        """Display the possible completions"""
-        if len(completions) == 0 or completions == [completion_text]:
-            return
-        self.completion_text = completion_text
-        self.show_completion_widget(sorted(completions, key=string.lower))
-
     # Methods implemented in child class:
-    # (e.g. InteractiveShell)
+    # (e.g. InternalShell)
     def get_dir(self, objtxt):
         """Return dir(object)"""
         raise NotImplementedError
@@ -948,7 +936,7 @@ class PythonShellWidget(ShellBaseWidget):
         """Return True if object is defined"""
         raise NotImplementedError
         
-    def show_code_completion(self):
+    def show_code_completion(self, automatic):
         """Display a completion list based on the current line"""
         # Note: unicode conversion is needed only for ExternalShellBase
         text = unicode(self.get_current_line_to_cursor())
@@ -960,19 +948,27 @@ class PythonShellWidget(ShellBaseWidget):
             try1 = text.split(' ')[-1]
             obj_list = self.get_completion(try1)
             if obj_list:
-                self.show_completion_list(obj_list, completion_text=try1)
+                self.show_completion_list(obj_list, completion_text=try1,
+                                          automatic=automatic)
                 return
+        elif text.startswith('%'):
+            # IPython magic commands
+            obj_list = self.get_completion(text)
+            if obj_list:
+                self.show_completion_list(obj_list, completion_text=text,
+                                          automatic=automatic)
+            # There is no point continuing the process when text starts with '%'
+            return
         obj_list = self.get_completion(last_obj)
         if not text.endswith('.') and last_obj and obj_list:
-            self.show_completion_list(obj_list, completion_text=last_obj)
+            self.show_completion_list(obj_list, completion_text=last_obj,
+                                      automatic=automatic)
             return
         #-----------------------------------------------------------------------
         
         obj_dir = self.get_dir(last_obj)
-        if last_obj and obj_dir:
-            if not text.endswith('.'):
-                return
-            self.show_completion_list(obj_dir)
+        if last_obj and obj_dir and text.endswith('.'):
+            self.show_completion_list(obj_dir, automatic=automatic)
             return
         
         # Builtins and globals
@@ -980,11 +976,13 @@ class PythonShellWidget(ShellBaseWidget):
         if not text.endswith('.') and last_obj \
            and re.match(r'[a-zA-Z_0-9]*$', last_obj):
             b_k_g = dir(__builtin__)+self.get_globals_keys()+keyword.kwlist
-            if last_obj in b_k_g:
-                return
             for objname in b_k_g:
-                if objname.startswith(last_obj):
-                    self.show_completion_list(b_k_g, completion_text=last_obj)
+                if objname.startswith(last_obj) and objname != last_obj:
+                    self.show_completion_list(b_k_g, completion_text=last_obj,
+                                              automatic=automatic)
+                    return
+            else:
+                return
         
         # Looking for an incomplete completion
         if last_obj is None:
@@ -999,42 +997,44 @@ class PythonShellWidget(ShellBaseWidget):
             completions = self.get_dir(last_obj)
             if completions is not None:
                 self.show_completion_list(completions,
-                                          completion_text=completion_text)
+                                          completion_text=completion_text,
+                                          automatic=automatic)
                 return
         
         # Looking for ' or ": filename completion
         q_pos = max([text.rfind("'"), text.rfind('"')])
         if q_pos != -1:
             self.show_completion_list(self.get_cdlistdir(),
-                                      completion_text=text[q_pos+1:])
+                                      completion_text=text[q_pos+1:],
+                                      automatic=automatic)
             return
     
     def show_docstring(self, text, call=False):
         """Show docstring or arguments"""
-        if not self.calltips:
-            return
-        
         text = unicode(text) # Useful only for ExternalShellBase
         
-        if (self.inspector is not None) and \
+        if self.inspector_enabled and (self.inspector is not None) and \
            (self.inspector.dockwidget.isVisible()):
             # ObjectInspector widget exists and is visible
-            self.inspector.set_object_text(text)
+            self.inspector.set_shell(self)
+            self.inspector.set_object_text(text, ignore_unknown=True)
             self.setFocus() # if inspector was not at top level, raising it to
                             # top will automatically give it focus because of
                             # the visibility_changed signal, so we must give
                             # focus back to shell
-            if call:
+            if call and self.calltips:
                 # Display argument list if this is function call
                 iscallable = self.iscallable(text)
                 if iscallable is not None:
                     if iscallable:
                         arglist = self.get_arglist(text)
+                        if isinstance(arglist, bool):
+                            arglist = []
                         if arglist:
                             self.show_calltip(translate("PythonShellWidget",
                                                         "Arguments"),
                                               arglist, '#129625')
-        else: # inspector is not visible
+        elif self.calltips: # inspector is not visible or link is disabled
             doc = self.get__doc__(text)
             if doc is not None:
                 self.show_calltip(translate("PythonShellWidget",
@@ -1052,7 +1052,22 @@ class PythonShellWidget(ShellBaseWidget):
         """Set ObjectInspector DockWidget reference"""
         self.inspector = inspector
         self.inspector.set_shell(self)
-            
+
+    def set_inspector_enabled(self, state):
+        self.inspector_enabled = state
+        
+    def inspect_current_object(self):
+        text = ''
+        text1 = self.get_text('sol', 'cursor')
+        tl1 = re.findall(r'([a-zA-Z_]+[0-9a-zA-Z_\.]*)', text1)
+        if tl1 and text1.endswith(tl1[-1]):
+            text += tl1[-1]
+        text2 = self.get_text('cursor', 'eol')
+        tl2 = re.findall(r'([0-9a-zA-Z_\.]+[0-9a-zA-Z_\.]*)', text2)
+        if tl2 and text2.startswith(tl2[0]):
+            text += tl2[0]
+        if text:
+            self.show_docstring(text)
             
     #------ Drag'n Drop
     def drop_pathlist(self, pathlist):
@@ -1088,9 +1103,9 @@ class TerminalWidget(ShellBaseWidget):
                 
     def _key_backspace(self, cursor_position):
         """Action for Backspace key"""
-        if self.hasSelectedText():
+        if self.has_selected_text():
             self.check_selection()
-            self.removeSelectedText()
+            self.remove_selected_text()
         elif self.current_prompt_pos == cursor_position:
             # Avoid deleting prompt
             return
@@ -1102,22 +1117,8 @@ class TerminalWidget(ShellBaseWidget):
         if self.is_cursor_on_last_line():
             self.stdkey_tab()
                 
-    def _key_home(self, shift):
-        """Action for Home key"""
-        if self.is_cursor_on_last_line():
-            self.stdkey_home(shift, self.current_prompt_pos)
-                
-    def _key_end(self, shift):
-        """Action for End key"""
-        if self.is_cursor_on_last_line():
-            self.stdkey_end(shift)
-                
-    def _key_pageup(self):
-        """Action for PageUp key"""
-        pass
-    
-    def _key_pagedown(self):
-        """Action for PageDown key"""
+    def _key_ctrl_space(self):
+        """Action for Ctrl+Space"""
         pass
                 
     def _key_escape(self):
