@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009 Pierre Raybaut
+# Copyright © 2009-2010 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -19,7 +19,9 @@ class TextEditor(QDialog):
     """Array Editor Dialog"""
     def __init__(self, text, title='', font=None, parent=None,
                  readonly=False, size=(400, 300)):
-        super(TextEditor, self).__init__(parent)
+        QDialog.__init__(self, parent)
+        
+        self._conv = str if isinstance(text, str) else unicode
         
         self.layout = QVBoxLayout()
         self.setLayout(self.layout)
@@ -50,21 +52,23 @@ class TextEditor(QDialog):
                             "%s" % (" - "+str(title) if str(title) else ""))
         self.resize(size[0], size[1])
         
-    def get_copy(self):
+    def get_value(self):
         """Return modified text"""
-        return unicode(self.edit.toPlainText())
+        return self._conv(self.edit.toPlainText())
     
     
 def test():
     """Text editor demo"""
     from spyderlib.utils.qthelpers import qapplication
-    _app = qapplication()
+    app = qapplication()
     dialog = TextEditor("""
     01234567890123456789012345678901234567890123456789012345678901234567890123456789
     dedekdh elkd ezd ekjd lekdj elkdfjelfjk e
     """)
-    if dialog.exec_():
-        text = dialog.get_copy()
+    dialog.show()
+    app.exec_()
+    if dialog.result():
+        text = dialog.get_value()
         print "Accepted:", text
         dialog = TextEditor(text)
         dialog.exec_()
